#$Id$#

from books.api.ContactsApi import ContactsApi
from books.api.ContactPersonsApi import ContactPersonsApi
from books.api.EstimatesApi import EstimatesApi
from books.api.InvoicesApi import InvoicesApi
from books.api.RecurringInvoicesApi import RecurringInvoicesApi
from books.api.CreditNotesApi import CreditNotesApi
from books.api.CustomerPaymentsApi import CustomerPaymentsApi
from books.api.ExpensesApi import ExpensesApi
from books.api.RecurringExpensesApi import RecurringExpensesApi
from books.api.BillsApi import BillsApi
from books.api.VendorPaymentsApi import VendorPaymentsApi
from books.api.BankAccountsApi import BankAccountsApi
from books.api.BankTransactionsApi import BankTransactionsApi
from books.api.BankRulesApi import BankRulesApi
from books.api.ChartOfAccountsApi import ChartOfAccountsApi
from books.api.JournalsApi import JournalsApi
from books.api.BaseCurrencyAdjustmentApi import BaseCurrencyAdjustmentApi
from books.api.ProjectsApi import ProjectsApi
from books.api.SettingsApi import SettingsApi
from books.api.ItemsApi import ItemsApi
from books.api.OrganizationsApi import OrganizationsApi
from books.api.UsersApi import UsersApi


class ZohoBooks:
    """
    This class is used to create an object for books service and to provide instance for all APIs.
    """
   
    def __init__(self, authtoken, organization_id):
        """Initialize the parameters for Zoho books.

        Args:
            authtoken(str): User's Authtoken.
            organization_id(str): User's Organization id.

        """
        self.authtoken=authtoken
        self.organization_id=organization_id
    
    def get_contacts_api(self):
        """Get instance for contacts api.
  
        Returns:
            instance: Contacts api instance.

        """
        contacts_api = ContactsApi(self.authtoken, self.organization_id)
        return contacts_api

    def get_contact_persons_api(self):
        """Get instance for contact persons api.
 
        Returns:
            instance: Contact persons api.

        """
        contact_persons_api = ContactPersonsApi(self.authtoken, 
                                                self.organization_id)
        return contact_persons_api
    
    def get_estimates_api(self):
        """Get instance for estimates api.
 
        Returns: 
            instance: Estimates api.

        """
        estimates_api = EstimatesApi(self.authtoken, self.organization_id)
        return estimates_api
    
    def get_invoices_api(self):
        """Get instance for invoice api.
 
        Returns:
            instance: Invoice api.

        """
        invoices_api = InvoicesApi(self.authtoken, self.organization_id)
        return invoices_api

    def get_recurring_invoices_api(self):
        """Get instance for recurring invoices api.

        Returns:
            instance: Recurring invoice api.

        """
        recurring_invoices_api = RecurringInvoicesApi(self.authtoken, \
                                                      self.organization_id)
        return recurring_invoices_api

    def get_creditnotes_api(self):
        """Get instance for creditnotes api.
 
        Returns:
            instance: Creditnotes api.

        """
        creditnotes_api = CreditNotesApi(self.authtoken, self.organization_id)
        return creditnotes_api

    def get_customer_payments_api(self):
        """Get instance for customer payments api.

        Returns:
            instance: Customer payments api.

        """
        customer_payments_api = CustomerPaymentsApi(self.authtoken, 
                                                    self.organization_id)
        return customer_payments_api
    
    def get_expenses_api(self):
        """Get instance for expenses api.
         
        Returns:
            instance: Expenses api.

        """
        expenses_api = ExpensesApi(self.authtoken, self.organization_id)
        return expenses_api
    
    def get_recurring_expenses_api(self):
        """Get instance for recurring expenses api.

        Returns:
            instance: Recurring expenses api.

        """
        recurring_expenses_api = RecurringExpensesApi(self.authtoken, 
                                                     self.organization_id)
        return recurring_expenses_api

    def get_bills_api(self):
        """Get instance for bills api.

        Returns:
            instance: Bills api

        """
        bills_api = BillsApi(self.authtoken, self.organization_id)
        return bills_api

    def get_vendor_payments_api(self):
        """Get instance for vendor payments api.

        Returns:
            instance: vendor payments api

        """
        vendor_payments_api = VendorPaymentsApi(self.authtoken, 
                                                self.organization_id)
        return vendor_payments_api
   
    def get_bank_accounts_api(self):
        """Get instancce for bank accounts api.

        Returns: 
            instance: Bank accounts api.

        """ 
        bank_accounts_api = BankAccountsApi(self.authtoken, 
                                            self.organization_id)
        return bank_accounts_api

    def get_bank_transactions_api(self):
        """Get instance for bank transactions api.

        Returns:
            instance: Bank Transactions api.

        """
        bank_transactions_api = BankTransactionsApi(self.authtoken,
                                                    self.organization_id)
        return bank_transactions_api

    def get_bank_rules_api(self):
        """Get instance for bank rules api.
      
        Returns:
            instance: Bank rules api.

        """
        bank_rules_api = BankRulesApi(self.authtoken, self.organization_id)
        return bank_rules_api
  
    def get_chart_of_accounts_api(self):
        """Get instancce for chart of accounts api

        Returns:
            instance: Chart of accounts api.

        """
        chart_of_accounts_api = ChartOfAccountsApi(self.authtoken, 
                                                   self.organization_id)
        return chart_of_accounts_api

    def get_journals_api(self):
        """Get instance for journals api.
     
        Returns:
            instance: Journals api.

        """
        journals_api = JournalsApi(self.authtoken, self.organization_id)
        return journals_api

    def get_base_currency_adjustment_api(self): 
        """Get instance for base currency adjustment api

        Returns: 
            instance: Base currency adjustments api.

        """
        base_currency_adjustment_api = BaseCurrencyAdjustmentApi(\
                                       self.authtoken, self.organization_id)
        return base_currency_adjustment_api

    def get_projects_api(self):
        """Get instance for projects api.
       
        Returns:
            instance: Projects api.

        """
        projects_api = ProjectsApi(self.authtoken, self.organization_id)
        return projects_api

    def get_settings_api(self):
        """Get instance for settings api.
      
        Returns:
            instance: Settings api.

        """
        settings_api = SettingsApi(self.authtoken, self.organization_id)
        return settings_api

    def get_items_api(self):
        """Get instance for items api.
 
        Returns: 
            instance: Items api.

        """
        items_api = ItemsApi(self.authtoken, self.organization_id)
        return items_api

    def get_users_api(self):
        """Get instance for users api.

        Returns:
            instance: Users api.

        """
        users_api = UsersApi(self.authtoken, self.organization_id)
        return users_api

    def get_organizations_api(self):
        """Get instance for organizations api.

        Returns:
            instance: Organizations api.

        """
        organizations_api = OrganizationsApi(self.authtoken, self.organization_id)
        return organizations_api



#$Id$#

from books.model.CreditNote import CreditNote
from books.model.PageContext import PageContext
from books.model.CreditNoteList import CreditNoteList
from books.model.LineItem import LineItem
from books.model.Address import Address
from books.model.EmailHistory import EmailHistory
from books.model.Email import Email
from books.model.EmailTemplate import EmailTemplate
from books.model.ToContact import ToContact
from books.model.FromEmail import FromEmail
from books.model.Template import Template
from books.model.InvoiceCredited import InvoiceCredited
from books.model.InvoiceCreditedList import InvoiceCreditedList
from books.model.Invoice import Invoice
from books.model.InvoiceList import InvoiceList
from books.model.CreditNoteRefund import CreditNoteRefund
from books.model.CreditNoteRefundList import CreditNoteRefundList
from books.model.Comment import Comment
from books.model.CommentList import CommentList

class CreditNotesParser:
    """This class is used to parse the json for credit notes."""
 
    def creditnotes_list(self,response):
        """This method parses the given response and creates a creditnotes 
            list object.

        Args:
            response(dict):Response containing json object for credit notes 
                list.

        Returns:
            instance: Creditnotes list object.

        """
        credit_notes_list=CreditNoteList()
        credit_notes=[]
        for value in response['creditnotes']:
            credit_note=CreditNote()
            credit_note.set_creditnote_id(value['creditnote_id'])
            credit_note.set_creditnote_number(value['creditnote_number'])
            credit_note.set_status(value['status'])
            credit_note.set_reference_number(value['reference_number'])
            credit_note.set_date(value['date'])
            credit_note.set_total(value['total'])
            credit_note.set_balance(value['balance'])
            credit_note.set_customer_id(value['customer_id'])
            credit_note.set_customer_name(value['customer_name'])
            credit_note.set_currency_id(value['currency_id'])
            credit_note.set_currency_code(value['currency_code'])
            credit_note.set_created_time(value['created_time'])
            credit_note.set_last_modified_time(value['last_modified_time'])
            credit_note.set_is_emailed(value['is_emailed'])
            credit_notes.append(credit_note)
        credit_notes_list.set_creditnotes(credit_notes)
        page_context_obj=PageContext()
        page_context=response['page_context']
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_applied_filter(page_context['applied_filter'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        credit_notes_list.set_page_context(page_context_obj)
        return credit_notes_list

    def get_creditnote(self,response):
        """This method parses the given response and returns credit note 
            object.

        Args:
            response(dict): Response containing json object for credit note.

        Returns:
            instance: Credit note object.

        """
        creditnote=response['creditnote']
        credit_note=CreditNote()
        credit_note.set_creditnote_id(creditnote['creditnote_id'])
        credit_note.set_creditnote_number(creditnote['creditnote_number'])
        credit_note.set_date(creditnote['date'])
        credit_note.set_status(creditnote['status'])
        credit_note.set_reference_number(creditnote['reference_number'])
        credit_note.set_customer_id(creditnote['customer_id'])
        credit_note.set_customer_name(creditnote['customer_name'])
        credit_note.set_contact_persons(creditnote['contact_persons'])
        credit_note.set_currency_id(creditnote['currency_id'])
        credit_note.set_currency_code(creditnote['currency_code'])
        credit_note.set_exchange_rate(creditnote['exchange_rate'])
        credit_note.set_price_precision(creditnote['price_precision'])
        credit_note.set_template_id(creditnote['template_id'])
        credit_note.set_template_name(creditnote['template_name'])
        credit_note.set_is_emailed(creditnote['is_emailed'])
        for value in creditnote['line_items']:
            line_item=LineItem()
            line_item.set_item_id(value['item_id'])
            line_item.set_line_item_id(value['line_item_id'])
            line_item.set_account_id(value['account_id'])
            line_item.set_account_name(value['account_name'])
            line_item.set_name(value['name'])
            line_item.set_description(value['description'])
            line_item.set_item_order(value['item_order'])
            line_item.set_quantity(value['quantity'])
            line_item.set_unit(value['unit'])
            line_item.set_bcy_rate(value['bcy_rate'])
            line_item.set_rate(value['rate'])
            line_item.set_tax_id(value['tax_id'])
            line_item.set_tax_name(value['tax_name'])
            line_item.set_tax_type(value['tax_type'])
            line_item.set_tax_percentage(value['tax_percentage'])
            line_item.set_item_total(value['item_total'])
            credit_note.set_line_items(line_item)
        credit_note.set_sub_total(creditnote['sub_total'])
        credit_note.set_total(creditnote['total'])
        credit_note.set_total_credits_used(creditnote['total_credits_used'])   
        credit_note.set_total_refunded_amount(\
        creditnote['total_refunded_amount'])
        credit_note.set_balance(creditnote['balance'])
        taxes=[]
        for value in creditnote['taxes']:
            tax=Tax()
            tax.set_tax_name(value['tax_name'])
            tax.set_tax_amount(value['tax_amount'])
            taxes.append(tax)
        credit_note.set_taxes(taxes)
        billing_address=creditnote['billing_address']
        billing_address_obj=Address()
        billing_address_obj.set_address(billing_address['address'])
        billing_address_obj.set_city(billing_address['city'])
        billing_address_obj.set_state(billing_address['state'])
        billing_address_obj.set_zip(billing_address['zip'])
        billing_address_obj.set_country(billing_address['country'])
        billing_address_obj.set_fax(billing_address['fax'])
        credit_note.set_billing_address(billing_address_obj)
        shipping_address=creditnote['shipping_address']
        shipping_address_obj=Address()
        shipping_address_obj.set_address(shipping_address['address'])
        shipping_address_obj.set_city(shipping_address['city'])
        shipping_address_obj.set_state(shipping_address['state'])
        shipping_address_obj.set_zip(shipping_address['zip'])
        shipping_address_obj.set_country(shipping_address['country'])
        shipping_address_obj.set_fax(shipping_address['fax'])
        credit_note.set_shipping_address(shipping_address_obj)
        credit_note.set_created_time(creditnote['created_time'])
        credit_note.set_last_modified_time(creditnote['last_modified_time'])
        return credit_note

    def get_message(self,response):
        """This method parses the json object and returns string message.

        Args:
            response(dict): Response containing json object.

        Returns:
            str: Success message.

        """
        return response['message']
 
    def email_history(self,response):
        """This method parses the json object and returns list of email \
            history object.

        Args:
            response(dict): Response containing json object for email history.

        Returns:
            list of instance: List of email history object.
  
        """
        email_history=[]
        for value in response['email_history']:
            email_history_obj=EmailHistory()
            email_history_obj.set_mailhistory_id(value['mailhistory_id'])
            email_history_obj.set_from(value['from'])
            email_history_obj.set_to_mail_ids(value['to_mail_ids'])
            email_history_obj.set_subject(value['subject'])
            email_history_obj.set_date(value['date'])
            email_history_obj.set_type(value['type'])
            email_history.append(email_history_obj)
        return email_history
 
    def email(self,response):
        """This method parses the response and returns email object.

        Args:
            response(dict):  Response containing json object for email.

        Returns:
            instance: Email object.

        """
        data=response['data']
        email=Email()
        email.set_body(data['body'])
        email.set_error_list(data['error_list'])
        email.set_subject(data['subject'])
        for value in data['emailtemplates']:
            email_template=EmailTemplate()
            email_template.set_selected(value['selected'])
            email_template.set_name(value['name'])
            email_template.set_email_template_id(value['email_template_id'])
            email.set_email_templates(email)
        for value in data['to_contacts']:
            to_contact=ToContact()
            to_contact.set_first_name(value['first_name'])
            to_contact.set_selected(value['selected'])
            to_contact.set_phone(value['phone'])
            to_contact.set_email(value['email'])
            to_contact.set_last_name(value['last_name'])
            to_contact.set_salutation(value['salutation'])
            to_contact.set_contact_person_id(value['contact_person_id'])
            to_contact.set_mobile(value['mobile'])
            email.set_to_contacts(to_contact)
        email.set_file_name(data['file_name'])
        email.set_file_name_without_extension(\
        data['file_name_without_extension'])
        for value in data['from_emails']:
            from_email=FromEmail()
            from_email.set_user_name(value['user_name'])
            from_email.set_selected(value['selected'])
            from_email.set_email(value['email'])
            from_email.set_is_org_email_id(value['is_org_email_id']) 
        email.set_from_emails(from_email)
        email.set_customer_id(data['customer_id'])
        return email

    def get_billing_address(self,response): 
        """This method parses the response containing json object for billing 
            address.

        Args:
            response(dict): Response containing json object for billing address.

        Returns:
            instance: Address object.

        """
        billing_address=response['billing_address']
        billing_address_obj=Address()
        billing_address_obj.set_address(billing_address['address'])
        billing_address_obj.set_city(billing_address['city'])
        billing_address_obj.set_state(billing_address['state'])
        billing_address_obj.set_zip(billing_address['zip'])
        billing_address_obj.set_country(billing_address['country'])
        billing_address_obj.set_fax(billing_address['fax'])
        billing_address_obj.set_is_update_customer(\
        billing_address['is_update_customer'])
        return billing_address_obj

    def get_shipping_address(self,response):
        """This method parses the json object and returns shipping address \
            object.

        Args:
            response(dict): Response containing json object for shipping \
                address.

        Returns:
            instance: Shipping address object.

        """
        shipping_address=response['shipping_address']
        shipping_address_obj=Address()
        shipping_address_obj.set_address(shipping_address['address'])    
        shipping_address_obj.set_city(shipping_address['city'])
        shipping_address_obj.set_state(shipping_address['state'])
        shipping_address_obj.set_zip(shipping_address['zip'])
        shipping_address_obj.set_country(shipping_address['country'])
        shipping_address_obj.set_fax(shipping_address['fax'])
        shipping_address_obj.set_is_update_customer(\
        shipping_address['is_update_customer'])
        return shipping_address_obj
 
    def list_templates(self,response):
        """This method parses the json object and returns templates list.

        Args:
            response(dict): Response containing json object for templates list.

        Returns:
            lsit of instance: Lsit of templates object.

        """
        templates=[]
        for value in response['templates']:
            template=Template()
            template.set_template_name(value['template_name'])
            template.set_template_id(value['template_id'])
            template.set_template_type(value['template_type'])
            templates.append(template)
        return templates

    def list_invoices_credited(self,response):
        """This method parses the json object and returns list of invoices 
            credited.

        Args:
            response(dict): Response containing json object for invoices 
                credited.

        Returns:
            instance: Invoices credited list object.

        """
        invoices_credited = InvoiceCreditedList()
        for value in response['invoices_credited']:
            invoice_credited = InvoiceCredited()
            invoice_credited.set_creditnote_id(value['creditnote_id'])
            invoice_credited.set_invoice_id(value['invoice_id'])
            invoice_credited.set_creditnotes_invoice_id(\
            value['creditnote_invoice_id'])
            invoice_credited.set_date(value['date'])
            invoice_credited.set_invoice_number(value['invoice_number'])
            invoice_credited.set_creditnotes_number(value['creditnote_number'])
            invoice_credited.set_credited_amount(value['credited_amount'])
            invoices_credited.set_invoices_credited(invoice_credited)
        return invoices_credited

    def credit_to_invoice(self,response):
        """This method parses the given response and returns invoices object.
 
        Args:
            response(dict): Responses containing json object for credit to 
                invoice.

        Returns:
            instance: Invoice List object.

        """
        invoices = InvoiceList()
        for value in response['apply_to_invoices']['invoices']:
            invoice=Invoice()
            invoice.set_invoice_id(value['invoice_id'])
            invoice.set_amount_applied(value['amount_applied'])
            invoices.set_invoices(invoice)
        return invoices

    def creditnote_refunds(self,response):
        """This method parses the given response and returns credit notes 
            refund list.

        Args:
            response(dict): Repsonse containing json object for credit notes 
                list.

        Returns:
            instance: Creditnote list object.

        """
        creditnotes_refund_list=CreditNoteRefundList()
        for value in response['creditnote_refunds']:
            creditnotes_refund=CreditNoteRefund()
            creditnotes_refund.set_creditnote_refund_id(\
            value['creditnote_refund_id'])
            creditnotes_refund.set_creditnote_id(value['creditnote_id'])   
            creditnotes_refund.set_date(value['date'])
            creditnotes_refund.set_refund_mode(value['refund_mode'])
            creditnotes_refund.set_reference_number(value['reference_number'])
            creditnotes_refund.set_creditnote_number(\
            value['creditnote_number'])
            creditnotes_refund.set_customer_name(value['customer_name'])
            creditnotes_refund.set_description(value['description'])
            creditnotes_refund.set_amount_bcy(value['amount_bcy'])
            creditnotes_refund.set_amount_fcy(value['amount_fcy'])
            creditnotes_refund_list.set_creditnote_refunds(creditnotes_refund)
        page_context=response['page_context']
        page_context_obj=PageContext()
        page_context_obj.set_page(page_context['page'])
        page_context_obj.set_per_page(page_context['per_page'])
        page_context_obj.set_has_more_page(page_context['has_more_page'])
        page_context_obj.set_report_name(page_context['report_name'])
        page_context_obj.set_sort_column(page_context['sort_column'])
        page_context_obj.set_sort_order(page_context['sort_order'])
        creditnotes_refund_list.set_page_context(page_context_obj)
        return creditnotes_refund_list

    def get_creditnote_refund(self,response):
        """This method parses the given repsonse and returns credit note 
            refund object.

        Args:
            response(dict): Response containing json object for credit note 
                refund.

        Returns:
            isntacne: Credit note object.

        """
        creditnote_refund=response['creditnote_refund']
        creditnote_refund_obj=CreditNoteRefund()
        creditnote_refund_obj.set_creditnote_refund_id(\
        creditnote_refund['creditnote_refund_id'])
        creditnote_refund_obj.set_creditnote_id(\
        creditnote_refund['creditnote_id'])
        creditnote_refund_obj.set_date(creditnote_refund['date'])
        creditnote_refund_obj.set_refund_mode(creditnote_refund['refund_mode'])
        creditnote_refund_obj.set_reference_number(\
        creditnote_refund['reference_number'])
        creditnote_refund_obj.set_amount(creditnote_refund['amount'])
        creditnote_refund_obj.set_exchange_rate(\
        creditnote_refund['exchange_rate'])
        creditnote_refund_obj.set_from_account_id(\
        creditnote_refund['from_account_id'])
        creditnote_refund_obj.set_from_account_name(\
        creditnote_refund['from_account_name'])
        creditnote_refund_obj.set_description(creditnote_refund['description'])
        return creditnote_refund_obj

    def comments_list(self,response):
        """This method parses the given response and returns the comments list.

        Args:
            response(dict): Response containing json object for comments list.
 
        Returns:
            list of instance: List of comments object.

        """ 
        comments_list = CommentList()
        for value in response['comments']:
            comment = Comment() 
            comment.set_comment_id(value['comment_id'])
            comment.set_creditnote_id(value['creditnote_id'])
            comment.set_description(value['description'])
            comment.set_commented_by_id(value['commented_by_id'])
            comment.set_commented_by(value['commented_by'])
            comment.set_comment_type(value['comment_type'])
            comment.set_date(value['date'])
            comment.set_date_description(value['date_description'])
            comment.set_time(value['time'])
            comment.set_operation_type(value['operation_type'])
            comment.set_transaction_id(value['transaction_id'])
            comment.set_transaction_type(value['transaction_type'])
            comments_list.set_comments(comment)
        return comments_list

    def get_comment(self,response):
        """This method parses the given response and returns comments object.

        Args:
            response(dict): Response containing json object for comments.

        Returns:
            instance: Comments object.

        """
        comment=response['comment']
        comment_obj=Comment()
        comment_obj.set_comment_id(comment['comment_id'])
        comment_obj.set_creditnote_id(comment['creditnote_id'])
        comment_obj.set_description(comment['description'])
        comment_obj.set_commented_by_id(comment['commented_by_id'])
        comment_obj.set_commented_by(comment['commented_by'])
        comment_obj.set_date(comment['date'])
        return comment_obj
     
      





















          



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.ChartOfAccountsParser import ChartOfAccountsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'chartofaccounts/'
parser = ChartOfAccountsParser()
zoho_http_client = ZohoHttpClient()

class ChartOfAccountsApi:
    """Chart of Accounts Api is used to:
     
    1.List chart of Accounts.
    2.Get the details of an account.
    3.Creates an account with the given account type.
    4.Updates an existing account.
    5.Delete an existing account.
    6.Update the account status as active.
    7.Update the account status as inactive.
    8.List all invoices transactions for the given account.
    9.Deletes the transaction.

    """
    
    def __init__(self, authtoken, organization_id):
        """Initilaize Chart of accounts api using user's authtoken and 
            organization id.

        Args: 
            authotoken(str): User's Authtoken.
            organization id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken,
            'organization_id': organization_id
            }

    def get_chart_of_accounts(self, parameters=None):
        """Get list of chart of accounts.

        Args:
            parameters(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Chart of accounts list object.

        """
        resp = zoho_http_client.get(base_url, self.details, parameters) 
        return parser.get_list(resp) 
  
    def get(self, account_id):    
        """Get details of an account.
 
        Args:
            account_id(str): Account id.

        Returns:
            instance: Chart of accounts object.

        """
        url = base_url + account_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_account(resp) 

    def create(self, account):
        """Create an account.

        Args:
            account(instance): Chart of accounts object.

        Returns:
            instance: Chart of accounts object.

        """
        json_object = dumps(account.to_json())
        data = { 
            'JSONString': json_object
            }
        resp = zoho_http_client.post(base_url, self.details, data)
        return parser.get_account(resp) 
  
    def update(self, account_id, account):
        """Update an account.

        Args: 
            account_id(str): Account id.
            account(instance): Chart of accounts object.

        Returns:
            instance: Chart of accounts object.

        """
        url = base_url + account_id
        json_object = dumps(account.to_json())
        data = { 
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_account(resp) 

    def delete(self, account_id):
        """Delete an account.

        Args:
            account_id(str): Account id.

        Returns:
            str: Success message('The account has been deleted.').

        """
        url = base_url + account_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
  
    def mark_an_account_as_active(self, account_id):
        """Mark an account as active.

        Args: 
            account_id(str): Account id.

        Returns: 
            str: Success message('The account has been marked as active.').

        """
        data = {
            'JSONString': ''
            }
        url = base_url + account_id + '/active'
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp)
 
    def mark_an_account_as_inactive(self, account_id):
        """Mark an account as inactive.
        
        Args:
            account_id(str): Account id.

        Returns:
            str: Success message('The account has been marked as inactive.').

        """
        data = {
            'JSONString': ''
            }
        url = base_url + account_id + '/inactive'
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp)

    def list_of_transactions(self, parameters=None):
        """List all involved transactions for a given account.

        Args:
            parameters(dict): Dictionary containing values for account id, date amount 
                filter_by, transaction type and sort column. 

        Returns:
            instance: Transaction list object.

        """
        url = base_url + 'transactions'
        resp = zoho_http_client.get(url, self.details, parameters) 
        return parser.get_transactions_list(resp) 

    def delete_a_transaction(self, transaction_id):
        """Delete the transaction.

        Args:
            transaction_id(str): Transaction id.

        Returns: 
            str: Success message('The transaction has been deleted.').

        """
        url = base_url + 'transactions/' + transaction_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 




#$Id$

class Customer:
    """This class is used to create object for customer."""
    def __init__(self):
        """Initialize parameters for organization object. """
        self.name = ''
        self.value = ''

    def set_name(self, name):
        """Set name.

        Args:
            name(str): Name.

        """
        self.name = name

    def get_name(self):
        """Get name.

        Returns:
            str: Name.

        """
        return self.name

    def set_value(self, value):
        """Set value.

        Args:
            value(str): Value.

        """
        self.value = value

    def get_value(self):
        """Get value.

        Returns:
            str: Value.

        """
        return self.value



#$Id$

from books.model.PageContext import PageContext

class CommentList: 
    """This class is used to create object for comments."""
    def __init__(self):
        """Initialize parameters for Comments list."""
        self.comments = []
        self.page_context = PageContext()

    def set_comments(self, comment):
        """Set comments.
      
        Args:
            comment(instance): Comment object.
        
        """
        self.comments.append(comment)
   
    def get_comments(self):
        """Get comments.
        
        Returns:
            list: List of comments object.
  
        """
        return self.comments

    def set_page_context(self, page_context):
        """Set page context.
        
        Args: 
            page_context(instance): Page context object.
        
        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.
    
        Returns:
            instance: Page context object.

        """
        return self.page_context


   



#$Id$

from books.model.PageContext import PageContext
from books.model.Instrumentation import Instrumentation

class TransactionList: 
    """This class is used to create object for Transaction list."""
    def __init__(self):
        """Initialize parameters for Transaction list object."""
        self.transactions = []
        self.page_context = PageContext()
        self.Instrumentation = Instrumentation()

    def set_transactions(self, transaction):
        """Set transactions.

        Args:
            transaction(instance): Transaction object.

        """
        self.transactions.append(transaction)

    def get_transactions(self):
        """Get transactions.

        Returns:
            list of instance: List of transactions object.

        """
        return self.transactions

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context  

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.
 
        Returns:
            instance: Page context object.

        """
        return self.page_context

    def set_instrumentation(self, instrumentation):
        """Set instrumentation.

        Args:
            instrumentation(instance): Instrumentation object.
     
        """
        self.instrumentation = instrumentation

    def get_instrumentation(self): 
        """Get instrumentation.

        Returns:
            instance: Instrumentation object.

        """
        return self.instrumentation

 



#$Id$

class Template:
    """This class is used to create object for templates."""
    def __init__(self):
        """Initialize the parameters for templates object."""
        self.template_name = ''
        self.template_id = ''
        self.template_type = ''
   
    def set_template_name(self, template_name):
        """Set template name.

        Args: 
            template_name(str): Template name.

        """
        self.template_name = template_name

    def get_template_name(self):
        """Get template name.
 
        Returns:
            str: Template name.

        """
        return self.template_name

    def set_template_id(self, template_id):
        """Set template id.

        Args: 
            template_id(str): Template id.

        """
        self.template_id = template_id

    def get_template_id(self):
        """Get template id.

        Returns:
            str: Template id.

        """
        return self.template_id

    def set_template_type(self, template_type):
        """Set template type.

        Args:
            template_type(str): Template type.
      
        """
        self.template_type = template_type

    def get_template_type(self):
        """Get template type.

        Returns:
            str: Template type.

        """
        return self.template_type





#$Id$

from books.model.PageContext import PageContext

class ItemList:
    """This class is used to create object for Items list."""
    def __init__(self):
        """Initialize parameters for Items list."""
        self.items = []
        self.page_context = PageContext()

    def set_items(self, item):
        """Set items.

        Args:
            item(instance): Item object.

        """
        self.items.append(item)

    def get_items(self):
        """Get items.

        Returns:
            list of instance: List of items object.

        """
        return self.items

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_context(instance): Page context object.
 
        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context



from setuptools import setup

setup(
    name='zohobooks-api',
    version='1.1',
    packages=['books', 'books.api', 'books.util', 'books.model', 'books.parser', 'books.service', 'books.exception'],
    url='',
    license='MIT',
    author='Sagar',
    author_email='sagarnrng17@gmail.com',
    description='Zoho Books Python Client Library'
)



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.SettingsParser import SettingsParser
from books.api.Api import Api
from json import dumps

base_url = Api().base_url + 'settings/'
zoho_http_client = ZohoHttpClient()
parser = SettingsParser()

class SettingsApi:
    """This class is used to 
 
    1.List preferences that are configured.
    2.Update the preferences that has been configured.
    3.Create a unit that can be associated to a line item.
    4.Delete a unit that has been associated to a line item.
    5.Get the details of invoice settings.
    6.Update the settings information of invoice.
    7.Get the details of notes and terms.
    8.Update invoice notes and terms.
    9.Get estimate settings.
    10.Update estimate settings.
    11.Get estimate notes and terms.
    12.Update estimate notes and terms.
    13.List creditnotes settings.
    14.Update creditnotes settings.
    15.Get creditnote notes and term
    16.Update creditnote notes and terms.
    17.List currencies.
    18.Get details of a currency.
    19.Create a currency.
    20.Update a currency.
    21.Delete a currency.
    22.List exchange rates.
    23.Get an exchange rate.
    24.Create an exchange rate.
    25.Update an exchange rate.
    26.Delete an exchange rate.
    27.List taxes.
    28.Get details of a tax.
    29.Create a tax.
    30.Update a tax.
    31.Delete a tax.
    32.Get a tax group.
    33.Create a tax group.
    34.Update a tax group.
    35.Delete a tax group.
    36.Get Opening balance.
    37.Create opening balance.
    38.Update opening balance.
    39.Delete opening balance.
    40.List auto payment reminder.
    41.Get an auto payment reminder.
    42.Enable auto reminder.
    43.Disable auto reminder.
    44.Update an auto reminder.
    45.List manual reminders.
    46.Get a manual reminder.
    47.Update a manual reminder. 

    """

    def __init__(self, authtoken, organization_id):
        """Initialize Settings Api using authtoken and organization id.

        Args:
            authtoken(str): User's Authtoken.
            organization_id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id 
            }
    
    def list_preferences(self):
        """List of preferences that are configured.

        Returns: 
            instance: Preference list object.

        """
        url = base_url + 'preferences'
        resp = zoho_http_client.get(url, self.details) 
        return parser.preference_list(resp) 

    def update_preferences(self, preference):
        """Update preference.

        Args:
            preference(instance): Preference object.

        Returns:
            str: Success message('Preferences have been saved.').
 
        """
        url = base_url + 'preferences'
        json_object = dumps(preference.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 

    def create_unit(self, unit):
        """Create a unit that can be associated to a line item.

        Args:
            unit(str): Unit. Eg: Kg. 
   
        Returns:
            str: Success message('Unit added.').

        """
        url = base_url + 'units'
        data = { 
            'unit': unit
            }
        json_string = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, json_string)
        return parser.get_message(resp) 
        
    def delete_unit(self, unit_id):
        """Delete a unit that has been associated to an item.

        Args:
            unit_id(str): Unit id.

        Returns:
            str: Success message('You have successfully deleted the unit.').

        """
        url = base_url + 'units/' + unit_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
    
    def get_invoice_settings(self):
        """Get the details of invoice settings.
 
        Returns:
            instance: Invoice settings object.
 
        """
        url = base_url + 'invoices'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_invoice_settings(resp) 

    def update_invoice_settings(self, invoice_settings):
        """Update the settings information for invoices.

        Args:
            invoice_settings(instance): Invoice settings.

        Returns:
            instance: Invoice settings object.

        """
        url = base_url + 'invoices'
        json_object = dumps(invoice_settings.to_json())
        data = { 
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_invoice_settings(resp)

    def get_invoice_notes_and_terms(self):
        """Get the details of invoce notes and terms.

        Returns:
            instance: Notes and terms object.

        """
        url = base_url + 'invoices/notesandterms'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_notes_and_terms(resp) 

    def update_invoice_notes_and_terms(self, invoice_notes_and_terms):
        """Update the notes and terms for an invoice.

        Args:
            invoice_notes_and_terms(instance): Invoice notes and terms object.

        Returns:
            instance: Invoice notes and terms object.

        """
        url = base_url + 'invoices/notesandterms'
        json_object = dumps(invoice_notes_and_terms.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_notes_and_terms(resp) 

    def get_estimate_settings(self):
        """Get estimate settings.

        Returns:
            instance: Estimate settings.

        """
        url = base_url + 'estimates'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_estimate_settings(resp) 

    def update_estimate_settings(self, estimate_setting):
        """Update estimate settings.

        Args:
            estimate_setting(instance): Estimate setting object.

        Returns:
            instance: Estiamte setting object.
 
        """
        url = base_url + 'estimates'
        json_object = dumps(estimate_setting.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_estimate_settings(resp) 

    def get_estimates_notes_and_terms(self): 
        """Get estimates notes and terms.

        Returns:   
            instance: notes and terms object.

        """
        url = base_url + 'estimates/notesandterms'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_notes_and_terms(resp) 

    def update_estimates_notes_and_terms(self, notes_and_terms):
        """Update estimate notes and terms.

        Args:
            notes_and_terms(instance): Notes and terms object.
 
        Returns:
            instance: Estimates notes and terms.
  
        """
        url = base_url + 'estimates/notesandterms'
        json_object = dumps(notes_and_terms.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data) 
        return parser.get_notes_and_terms(resp) 

    def list_creditnote_settings(self):
        """List creditnotes settings.

        Returns:
            instance: creditnotes settings.

        """
        url = base_url + 'creditnotes'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_creditnote_settings(resp) 

    def update_creditnote_settings(self, creditnotes_settings):
        """Update creditnotes settings.

        Args:
            creditnotes_settings(instance): Creditnotes settings object.

        Returns:
            instance: Creditnotes settings object.

        """
        url = base_url + 'creditnotes'
        json_object = dumps(creditnotes_settings.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_creditnote_settings(resp) 

    def get_creditnote_notes_and_terms(self):
        """Get creditnotes notes and terms.

        Returns:
            instance: Creditnotes settings object.

        """
        url = base_url + 'creditnotes/notesandterms'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_notes_and_terms(resp)

    def update_creditnote_notes_and_terms(self, notes_and_terms):
        """Update creditnotes notes and terms.

        Args:
            notes_and_terms(instance): Notes and terms object.

        Returns:
            instance: Notes and terms object.

        """
        url = base_url + 'creditnotes/notesandterms'
        json_object = dumps(notes_and_terms.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data) 
        return parser.get_notes_and_terms(resp) 

    def get_currencies(self, param=None):
        """List of configured currencies with pagination.

        Args:
            param(dict, optional): Filter with which the list has to be 
                 displayed.

        Returns:
            instance: Currency list object.

        """
        url = base_url + 'currencies'
        if param is not None:
            data = {
                'filter_by': param
                }
        else:
            data = None
        resp = zoho_http_client.get(url, self.details, data)
        return parser.get_currencies(resp) 

    def get_currency(self, currency_id):
        """Get the details of a currency.

        Args:
            currency_id(str): Currency id.

        Returns:
            instance: Currency object.

        """
        url = base_url + 'currencies/' + currency_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_currency(resp) 

    def create_currency(self, currency):
        """Create a currency for transactions.

        Args:
            currency(instance): Currency object.

        Returns: 
            instance: Currency object.

        """
        url = base_url + 'currencies'
        json_obj = dumps(currency.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_currency(resp) 
    
    def update_currency(self, currency_id, currency):
        """Update the details of currency.

        Args:
            currency_id(str): Currency id.
            currency(instance): Currency object.

        Returns:     
            instance: Currecny object.

        """
        url = base_url + 'currencies/' + currency_id 
        json_obj = dumps(currency.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_currency(resp) 
     
    def delete_currency(self, currency_id):
        """Delete currency.

        Args:
            currency_id(str): Currency id.

        Returns:
            str: Success message('The currency has been deleted.').
 
        """
        url = base_url + 'currencies/' + currency_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def list_exchange_rates(self, currency_id, param=None):
        """List of exchange rates configured for the currency.

        Args:
            param(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Exchange rate list object.

        """
        url = base_url + 'currencies/' + currency_id + '/exchangerates'
        resp = zoho_http_client.get(url, self.details, param)
        return parser.get_exchange_rates(resp) 

    def get_exchange_rate(self, currency_id, exchange_rate_id):
        """Get details of an exchange rate that has been associated to the 
            currency.

        Args:
            currency_id(str): Currency id.
            exchange_rate_id(str): Exchange rate id.

        Returns:
            instance: Exchange rate object.

        """ 
        url = base_url + 'currencies/' + currency_id + '/exchangerates/' + \
              exchange_rate_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_exchange_rate(resp) 

    def create_exchange_rate(self, exchange_rate):
        """Create an exchange rate for the specified currency.

        Args:
            currency_id(str): Currency id.
            exchange_rate(instance): Exchange rate object.

        Returns:
            instance: Exchange rate object.

        """
        url = base_url + 'currencies/' + exchange_rate.get_currency_id() + \
              '/exchangerates'
        json_obj = dumps(exchange_rate.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_exchange_rate(resp) 
 
    def update_exchange_rate(self, exchange_rate):
        """Update the details of exchange rate currency.

        Args: 
            exchange_rate(instance): Exchnage rate object.
           
        Returns:
            str: Success message('The exchange rate has been update.').
      
        """
        url = base_url + 'currencies/' + exchange_rate.get_currency_id() + \
              '/exchangerates/' + exchange_rate.get_exchange_rate_id()
        json_obj = dumps(exchange_rate.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 

    def delete_exchange_rate(self, currency_id, exchange_rate_id):
        """Delete an exchnage rate for the specified currency.

        Args:
            currency_id(str): Currency id.
            exchange_rate_id(str): Exchange rate id.

        Returns:
            str: Success message('Exchange rate successfully deleted.').
 
        """
        url = base_url + 'currencies/' + currency_id + '/exchangerates/' + \
              exchange_rate_id
        resp = zoho_http_client.delete(url, self.details) 
        return parser.get_message(resp) 

    def get_taxes(self):
        """List of taxes with pagination.

        Returns:
            instance: Tax list object.

        """
        url = base_url + 'taxes'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_taxes(resp) 

    def get_tax(self, tax_id):
        """Get details of a tax.

        Args:
            tax_id(str): Tax id.

        Returns:
            instance: Tax object.

        """
        url = base_url + 'taxes/' + tax_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_tax(resp) 
    
    def create_tax(self, tax):
        """Create tax.

        Args:
            tax(instance): Tax object.

        Returns:
            instance: Tax object.

        """
        url = base_url + 'taxes'
        json_obj = dumps(tax.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_tax(resp) 
 
    def update_tax(self, tax_id, tax):
        """Update the details of tax.

        Args:
            tax_id(str): Tax id.
            tax(instance): Tax object.

        Returns:
            instance: Tax object.

        """
        url = base_url + 'taxes/' + tax_id
        json_obj = dumps(tax.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_tax(resp) 
            
    def delete_tax(self, tax_id):
        """Delete tax.

        Args:
            tax_id(str): Tax id.

        Returns:
            str: Success message('The record has been deleted.').

        """
        url = base_url + 'taxes/' + tax_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
   
    def get_tax_group(self, tax_group_id):
        """Get details of a tax group with associated taxes.

        Args:
            tax_group_id(str): Tax group id.

        Returns:
            instance: Tax group object.

        """
        url = base_url + 'taxgroups/' + tax_group_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_tax_group(resp)

    def create_tax_group(self, tax_group):
        """Create a tax group associating multiple taxes.

        Args:
            tax_group_name(str): Tax group name.
            taxes(str): List of tax ids associated to the tax group.

        Returns:
            instance: Tax group object.
 
        """
        url = base_url + 'taxgroups'
        json_obj = dumps(tax_group.to_json())
        data = {
            'JSONString': json_obj
            }
        print(data)
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_tax_group(resp) 

    def update_tax_group(self, tax_group):
        """Update tax group.

        Args:
            tax_group_id(str): Tax group id.
            tax_group_name(str): Tax group name.
            taxes(str): List of tax ids associated to that tax group.

        Returns:
            instance: Tax group object.

        """
        url = base_url + 'taxgroups/' + tax_group.get_tax_group_id()
        json_obj = dumps(tax_group.to_json())
        data = {
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 
 
    def delete_tax_group(self, tax_group_id): 
        """Delete tax group.

        Args:
            tax_group_id(str): Tax group id.

        Returns:
            str: Success message('The tax group has been deleted.'). 
   
        """
        url = base_url + 'taxgroups/' + tax_group_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def get_opening_balance(self):
        """Get opening balance.

        Returns:
            instance: Opening balance object.

        """
        url = base_url + 'openingbalances'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_opening_balance(resp) 

    def create_opening_balance(self, opening_balance):
        """Create opening balance.

        Args:
            opening_balance(instance): Opening balance object.

        Returns:
            instance: Opening balance object.

        """
        url = base_url + 'openingbalances'
        json_object = dumps(opening_balance.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_opening_balance(resp) 

    def update_opening_balance(self, opening_balance):
        """Update opening balance.

        Args:
            opening_balance(instance): Opening balance object.

        Returns:
            instance: Opening balance object.

        """
        url = base_url + 'openingbalances'
        json_object = dumps(opening_balance.to_json())
        data = {
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_opening_balance(resp) 

    def delete_opening_balance(self):
        """Delete the entered opening balance.

        Returns:
            str: Success message('The entered opening balance has been 
                deleted.').

        """
        url = base_url + 'openingbalances'
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
                 
    def list_auto_payment_reminder(self):
        """List auto payment reminder.

        Returns:
            instance: Autoreminders list object.

        """
        url = base_url + 'autoreminders'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_autoreminders(resp) 

    def get_auto_payment_reminder(self, reminder_id):
        """Get auto payment reminder.
 
        Args:
            reminder_id(str): Reminder id.

        Returns:
            instance: Auto payment reminder object.

        """
        url = base_url + 'autoreminders/' + reminder_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_autoreminder(resp) 

    def enable_auto_reminder(self, reminder_id):
        """Enable automated payment reminder.

        Args:
            reminder_id(str): Reminder id.

        Returns:
            str: Success message('Payment reminder has been enabled.').

        """
        url = base_url + 'autoreminders/' + reminder_id + '/enable'
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 

    def disable_auto_reminder(self, reminder_id):
        """Disable automated payment reminder.

        Args:
            reminder_id(str): Reminder id.

        Returns:
            str: Success message('Payment reminder has been disabled.').

        """
        url = base_url + 'autoreminders/' + reminder_id + '/disable'
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data)
        return parser.get_message(resp) 

    def update_auto_reminder(self, reminder_id, autoreminder):
        """Update an auto reminder.

        Args: 
            reminder_id(str): Reminder id.
            autoreminder(instance): Auto reminder object.

        Returns:
            str: Success message('Your payment reminder preferences have 
                been saved.').

        """
        url = base_url + 'autoreminders/' + reminder_id 
        json_obj = autoreminder.to_json()
        data = { 
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 

    def list_manual_reminders(self, type_of_reminder=None):
        """List of manual reminders.

        Args:
            type_of_reminder(str): Type to select between open or overdue 
                reminder.

        Returns:
            instance: Manual reminder list object.

        """
        url = base_url + 'manualreminders'
        if type_of_reminder is not None:
           param = {
                'type': type_of_reminder
                }
        else:
            param = None
        resp = zoho_http_client.get(url, self.details)
        return parser.get_manual_reminders(resp) 

    def get_manual_reminder(self, reminder_id):
        """Get the details of a manual reminder.

        Args:
            manual_reminder(instance): Manual reminder object.

        Returns:
            instance: Manual reminder object.

        """
        url = base_url + 'manualreminders/' + reminder_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_manual_reminder(resp) 

    def update_manual_reminder(self, reminder_id, manual_reminder):
        """Update manual reminder.

        Args:
            reminder_id(str): Reminder id.
            manual_reminder(instance): Manual reminder.

        Returns:
            instance: Manual reminder.

        """
        url = base_url + 'manualreminders/' + reminder_id
        json_obj = dumps(manual_reminder.to_json())
        data = { 
            'JSONString': json_obj
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 
 



       




#$Id$

from books.model.Address import Address

class RecurringInvoice:
    """This class is used to creat object for Recurring invoice."""
    def __init__(self):
        """Initialize parameters for Recurring invoice object."""
        self.recurrence_name = ''
        self.customer_id = ''
        self.contact_persons = []
        self.template_id = ''
        self.start_date = ''
        self.end_date = ''
        self.recurrence_frequency = ''
        self.repeat_every = 0
        self.payment_terms = 0
        self.payment_terms_label = ''
        self.exchange_rate = 0.0
        self.payment_options = {
            'payment_gateways': []
            }
        self.discount = ''
        self.is_discount_before_tax = ''
        self.discount_type = ''
        self.line_items = []
        self.salesperson_name = ''
        self.shipping_charge = 0.0
        self.adjustment = 0.0
        self.adjustment_description = ''
        self.notes = ''
        self.terms = ''
        self.recurring_invoice_id = ''
        self.status = ''
        self.total = 0.0
        self.customer_name = ''
        self.last_sent_date = ''
        self.next_invoice_date = ''
        self.created_time = ''
        self.last_modified_time = ''
        self.currency_id = ''
        self.price_precision = 0
        self.currency_code = ''
        self.currency_symbol = ''
        self.late_fee = {}
        self.sub_total = 0.0
        self.tax_total = 0.0
        self.allow_partial_payments = None
        self.taxes = []
        self.billing_address = Address()
        self.shipping_address = Address()
        self.template_name = ''
        self.salesperson_id = ''
        self.custom_fields = []

    def set_custom_fields(self, custom_fields):
        """Set custom fields.

        Args:
            custom_fields(list): List of custom fields.

        """
        self.custom_fields.extend(custom_fields)

    def get_custom_fields(self):
        """Get custom fields.

        Returns:
            list: List of custom fields.

        """
        return self.custom_fields

    def set_currency_id(self, currency_id):
        """Set currency id.

        Args: 
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id
    
    def set_recurrence_name(self, recurrence_name):
        """Set name of the recurring invoice.

        Args:
            recurrence_name(str): Name of the recurring invoice.

        """
        self.recurrence_name = recurrence_name

    def get_recurrence_name(self):
        """Get name of the recurring invoice.

        Returns:
            str: Name of the recurring invoice..

        """
        return self.recurrence_name
  
    def set_customer_id(self, customer_id):
        """Set ID for the customer for which recurring invoice has been 
            created.

        Args: 
            customer_id(str): Customer id for which recurring invoice has been 
                created.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get ID for the customer for which recurring invoice has been 
            created.

        Returns:
            str: Customer id for which recurring invoice has been created.

        """
        return self.customer_id
   
    def set_contact_persons(self, contact_persons):
        """Set Contact persons associated with the recurring invoice.

        Args:
            contact_persons(list): List of contact persons id.

        """ 
        self.contact_persons.extend(contact_persons)

    def get_contact_persons(self):
        """Get contact persons associated with the recurring invoice.

        Returns:
            list: List of contact persons id.

        """
        return self.contact_persons

    def set_template_id(self, template_id):
        """Set ID of the pdf template associated with the recurring profile.

        Args:
            template_id(str): Template id.

        """
        self.template_id = template_id

    def get_template_id(self):
        """Get ID of the pdf template associated with the recurring profile.

        Returns:
            str: Template id.
 
        """
        return self.template_id
  
    def set_start_date(self, start_date):
        """Set start date for the recurring invoice.

        Args:
            start_date(str): Starting date of the recurring invoice.

        """
        self.start_date = start_date

    def get_start_date(self):
        """Get start date of the recurring invoice.

        Returns:
            str: Starting date of the recurring invoice.

        """
        return self.start_date

    def set_end_date(self, end_date):
        """Set the date at which the recurring invoice has to be expires.

        Args:
            end_date(str): Date on which recurring invoice expires.

        """
        self.end_date = end_date

    def get_end_date(self):
        """Get the date at which the recurring invoice has to be expires.

        Returns:
            str: Date on which recurring invoice expires.

        """
        return self.end_date
   
    def set_recurrence_frequency(self, recurrence_frequency):
        """Set the frequency for the recurring invoice.

        Args:
            recurrence_frequency(str): Frequency for the recurring invoice. 
                Allowed values re days, weeks, months and years.

        """
        self.recurrence_frequency = recurrence_frequency

    def get_recurrence_frequency(self):
        """Get the frequency of the recurring invoice.

        Returns:
            str: Frequency of the recurring invoice.

        """
        return self.recurrence_frequency
  
    def set_repeat_every(self, repeat_every):
        """Set frequency to repeat.

        Args:
            repeat_every(int): Frequency to repeat.

        """
        self.repeat_every = repeat_every

    def get_repeat_every(self):
        """Get frequency to repeat.

        Returns:
            int: Frequency to repeat.

        """
        return self.repeat_every
  
    def set_payment_terms(self, payment_terms):
        """Set payment terms in days. Invoice due date will be calculated 
            based on this.

        Args:
            payment_terms(int): Payment terms in days.Example 15, 30, 60.

        """
        self.payment_terms = payment_terms

    def get_payment_terms(self):
        """Get payment terms.
 
        Returns:
            int: Payment terms in days.

        """
        return self.payment_terms
   
    def set_payment_terms_label(self, payment_terms_label):
        """Set payment terms label.

        Args:
            payment_terms_label(str): Payment terms label. Default value for 15 
                days is "Net 15".
 
        """
        self.payment_terms_label = payment_terms_label

    def get_payment_terms_label(self):
        """Get payment terms label.
        
        Returns:
            str: Payment terms label.
        
        """
        return self.payment_terms_label
  
    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate for the currency.

        Args:
            exchange_rate(float): Exchange rate for currency.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate of the currency.

        Returns:
            float: Exchange rate of the currency.
  
        """
        return self.exchange_rate

    def set_payment_options(self, payment_gateways):
        """Set payment options associated with the recurring invoice, online 
            payment gateways.
         
        Args: 
            payment_gateways(instance): Online payment gateway through which 
                payment can be made.
        
        """ 
        self.payment_options['payment_gateways'].append(payment_gateways)
 
    def get_payment_options(self):
        """Get payment options associated with the recurring invoice, online 
            payment gateways.
   
        Returns:
            dict: Payment options details.

        """   
        return self.payment_options

    def set_discount(self, discount):
        """Set discount applied to the invoice.

        Args:
            discount(str): Discount applied to the invoice. It can be either 
                in % or in amount.

        """
        self.discount = discount

    def get_discount(self):
        """Get discount applied to the invoice.

        Returns:
            str: Discount applied to the invoice.

        """
        return self.discount
   
    def set_is_discount_before_tax(self, is_discount_before_tax):
        """Set whether to discount before tax.

        Args:
            is_discount_before_tax(bool): True to discount befroe tax else 
                False.

        """
        self.is_discount_before_tax = is_discount_before_tax

    def get_is_discount_before_tax(self):
        """Get whether to discount before tax.

        Returns:
            bool: True to discount before tax else False.

        """
        return self.is_discount_before_tax

    def set_discount_type(self, discount_type):
        """Set how the discount is specified.

        Args:
            discount_type(str): Discount type. Allowed values are entity_level 
                and item_level.

        """
        self.discount_type = discount_type

    def get_discount_type(self):
        """Get discount type.

        Returns:
            str: Discount type.

        """
        return self.discount_type
 
    def set_line_items(self, line_item):
        """Set line items for an invoice.

        Args:
            line_items(instance): Line items object.

        """
        self.line_items.append(line_item)

    def get_line_items(self):
        """Get line items of an invoice.

        Returns:
            list: List of line items instance.

        """
        return self.line_items

    def set_salesperson_name(self, salesperson_name):
        """Set name of the salesperson.

        Args: 
            salesperson_name(str): Salesperson name.

        """
        self.salesperson_name = salesperson_name

    def get_salesperson_name(self):
        """Get salesperson name.

        Returns:
            str: Salesperson name.

        """
        return self.salesperson_name
 
    def set_shipping_charge(self, shipping_charge):
        """Set shipping charges applied to the invoice.

        Args: 
            shipping_charge(float): Shipping charges applied to the invoice.

        """
        self.shipping_charge = shipping_charge

    def get_shipping_charge(self):
        """Get shipping charges applied to invoice.

        Returns:
            float: Shipping charges applied to the invoice.

        """
        return self.shipping_charge
  
    def set_adjustment(self, adjustment):
        """Set adjustments made to the invoice.
 
        Args:
            adjustment(float): Adjustments made to the invoice.

        """
        self.adjustment = adjustment

    def get_adjustment(self):
        """Get adjustments made to the invoice.

        Returns:
            float: Adjustments made to the invoice.

        """
        return self.adjustment
 
    def set_adjustment_description(self, adjustment_description):
        """Set the adjustment description.

        Args:
            adjustment_description(str): Adjustment description.

        """
        self.adjustment_description = adjustment_description

    def get_adjustment_description(self):
        """Get adjustment description.

        Returns:
            str: Adjustment description.

        """
        return self.adjustment_description

    def set_notes(self, notes):
        """Set notes.

        Args:
            notes(str): Notes.

        """
        self.notes = notes
 
    def get_notes(self):
        """Get notes.

        Returns:
            str: Notes.

        """
        return self.notes

    def set_terms(self, terms):
        """Set terms.

        Args:
            terms(str): Terms.

        """ 
        self.terms = terms
 
    def get_terms(self):
        """Get terms.

        Returns:
            str: Terms.

        """
        return self.terms

    def set_recurring_invoice_id(self, recurring_invoice_id):
        """Set recurring invoice id.

        Args:
            recurring_invoice_id(str): Recurring invoice id.

        """
        self.recurring_invoice_id = recurring_invoice_id
 
    def get_recurring_invoice_id(self):
        """Get recurring invoice id.
 
        Returns: 
            str: Recurring invoice id.

        """
        return self.recurring_invoice_id

    def set_status(self, status):
        """Set status.
       
        Args: 
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status
  
    def set_total(self, total):
        """Set total.

        Args:
            total(float): Total.
 
        """
        self.total = total

    def get_total(self):
        """Get total.
      
        Returns:
            float: Total.

        """
        return self.total

    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_last_sent_date(self, last_sent_date):
        """Set last sent date.

        Args:
            last_sent_date(str): Last sent date.

        """
        self.last_sent_date = last_sent_date

    def get_last_sent_date(self):
        """Get last sent date.

        Returns:
            str: Last sent date.

        """
        return self.last_sent_date

    def set_next_invoice_date(self, next_invoice_date):
        """Set next invoice date.

        Args: 
            next_invoice_date(str): Next invoice date.

        """
        self.next_invoice_date = next_invoice_date
 
    def get_next_invoice_date(self):
        """Get next invoice date.

        Returns:
            str: Next invoice date.

        """
        return self.next_invoice_date
   
    def set_created_time(self, created_time):
        """Set created time.
   
        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self): 
        """Get created time.

        Returns: 
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args:
            last_modified_time(str): Last modified time.

        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: Last modified time.

        """
        return self.last_modified_time

    def set_price_precision(self, price_precision):
        """Set price precision.
 
        Args:
            price_precision(int): Price precision.
 
        """
        self.price_precision = price_precision

    def get_price_precision(self):
        """Get price precision.

        Returns:
            int: Price precision.

        """
        return self.price_precision
  
    def set_currency_code(self, currency_code):
        """Set currency code.

        Args: 
            currency_code(str):  Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.
        """
        return self.currency_code
   
    def set_currency_symbol(self, currency_symbol):
        """Set currency symbol.

        Args:
            currency_symbol(str): Currency symbol.

        """
        self.currency_symbol = currency_symbol
  
    def get_currency_symbol(self):
        """Get currency symbol.

        Returns:
            str: Currency symbol.

        """
        return self.currency_symbol

    def set_late_fee(self, late_fee):
        """Set late fee.

        Args:
            late_fee(dict): Late fee.

        """
        self.late_fee = late_fee

    def get_late_fee(self):
        """Get late fee.
       
        Returns:
            dict: LAte fee.
        
        """
        return self.late_fee

    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.

        """
        self.sub_total = sub_total

    def get_sub_total(self):
        """Get sub total.

        Returns:
            float: Sub total.

        """
        return self.sub_total

    def set_tax_total(self, tax_total):
        """Set tax total.

        Args:
            tax_total(float): Tax total.

        """
        self.tax_total = tax_total

    def get_tax_total(self):
        """Get tax total.

        Returns:
            float: Tax total.

        """
        return self.tax_total
    
    def set_allow_partial_payments(self, allow_partial_payments):
        """Set whether to allow partial payments or not.

        Args:
            allow_partial_payments(bool): True to allow partial payments.

        """
        self.allow_partial_payments = allow_partial_payments

    def get_allow_partial_payments(self):
        """Get whether partial payments are allowed or not.

        Returns:
            str: True if partial payments are allowed else False.

        """ 
        return self.allow_partial_payments
   
    def set_taxes(self, taxes): 
        """Set taxes.

        Args:
            taxes(instance): Tax object.

        """
        self.taxes.append(taxes)

    def get_taxes(self):
        """Get taxes.

        Returns:
            list: List of taxes object.

        """
        return self.taxes

    def set_billing_address(self, address):
        """Set billing address.

        Args:
            address(instance): Address object.

        """
        self.billing_address = (address)
 
    def get_billing_address(self):
        """Get billing address.

        Returns: 
            instance: Billing address object.

        """
        return self.billing_address

    def set_shipping_address(self, address):
        """Set shipping address.
  
        Args:
            address(instance): Shipping address.

        """
        self.shipping_address = address

    def get_shipping_address(slef):
        """Get shipping address.

        Returns:
            instance: Shipping address object.

        """
        return self.shipping_address
   
    def set_template_name(self, template_name):
        """Set template name.

        Args: 
            template_name(str): Template name.

        """
        self.template_name = template_name
 
    def get_template_name(self):
        """Get template name.

        Returns:
            str: Template name.

        """
        return self.template_name


    def set_salesperson_id(self, salesperson_id):
        """Set salesperson id.
  
        Args: 
            salesperson_id(str): Salesperson id.

        """
        self.salesperson_id = salesperson_id

    def get_salesperson_id(self):
        """Get salesperson id.

        Returns:
            str: Salesperson id.

        """
        return self.salesperson_id

    def to_json(self):
        """This method is used to convert recurring invoice object to json object.

        Returns:
            dict: Dictionary containing json object for recurring invoice.

        """
        data = {}
        if self.recurrence_name != '':
            data['recurrence_name'] = self.recurrence_name
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.contact_persons:
            data['contact_persons'] = self.contact_persons
        if self.template_id != '':
            data['template_id'] = self.template_id
        if self.start_date != '':
            data['start_date'] = self.start_date
        if self.end_date != '':
            data['end_date'] = self.end_date
        if self.recurrence_frequency != '':
            data['recurrence_frequency'] = self.recurrence_frequency
        if self.repeat_every != '':
            data['repeat_every'] = self.repeat_every
        if self.payment_terms != '':
            data['payment_terms'] = self.payment_terms
        if self.payment_terms_label != '':
            data['payment_terms_label'] = self.payment_terms_label
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.payment_options['payment_gateways']:
            data['payment_options'] = {
                'payment_gateways': []
                }
            for value in self.payment_options['payment_gateways']:
                data['payment_options']['payment_gateways'].append(\
                                        value.to_json())
        if self.discount > 0:
            data['discount'] = self.discount
        if self.is_discount_before_tax is not None:
            data['is_discount_before_tax'] = self.is_discount_before_tax
        if self.discount_type != '':
            data['discount_type'] = self.discount_type
        if self.line_items:
            data['line_items'] = []
            for value in self.line_items:
                line_item = value.to_json()
                data['line_items'].append(line_item)
        if self.notes != '':
            data['notes'] = self.notes
        if self.terms != '':
            data['terms'] = self.terms
        if self.salesperson_name != '':
            data['salesperson_name'] = self.salesperson_name
        if self.shipping_charge > 0:
            data['shipping_charge'] = self.shipping_charge
        if self.adjustment != '':
            data['adjustment'] = self.adjustment
        if self.adjustment_description != '':
            data['adjustment_description'] = self.adjustment_description
        return data



#$Id$

from books.model.PageContext import PageContext

class ProjectList:
    """This class is used to create objects for projects list."""
    def __init__(self): 
        """Initialize parameters for Projects list."""
        self.projects = []
        self.page_context = PageContext()

    def set_projects(self, project):
        """Set projects.

        Args:
            project(instance): Project object.

        """
        self.projects.append(project)

    def get_projects(self):
        """Get projects.

        Returns:
            list of instance: List of projects object.

        """
        return self.projects

    def set_page_context(self, page_context):
        """Set page context.

        Args:
            page_cotext(instance): Page context object.

        """
        self.page_context = page_context

    def get_page_context(self):
        """Get page context.

        Returns:
            instance: Page context object.

        """
        return self.page_context
  



#$Id$

class Task: 
    """This class is used to create object for tasks."""
    def __init__(self): 
        """Initialize parameters for tasks object."""
        self.task_id = ''
        self.task_name = ''
        self.description = ''
        self.rate = 0.0
        self.budget_hours = 0
        self.total_hours = ''
        self.billed_hours= ''
        self.unbilled_hours = ''
        self.project_id = ''
        self.project_name = ''
        self.currency_id = ''
        self.customer_id = ''
        self.customer_name = ''
        self.log_time = ''

    def set_task_id(self, task_id):
        """Set task id.

        Args:
            task_id(str): Task id.

        """
        self.task_id = task_id

    def get_task_id(self):
        """Get task id.

        Returns:
            str: Task id.

        """
        return self.task_id

    def set_task_name(self, task_name):
        """Set task name.

        Args:
            task_name(str): Task name.

        """
        self.task_name = task_name

    def get_task_name(self):
        """Get task name.
       
        Returns:
            str: Task name.
 
        """
        return self.task_name

    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self):
        """Get description

        Returns:
            str: Description.

        """
        return self.description
    
    def set_rate(self, rate):
        """Set rate.
 
        Args:
            rate(float): Rate.

        """
        self.rate = rate

    def get_rate(self):
        """Get rate.

        Returns:
            float: Rate.

        """
        return self.rate

    def set_budget_hours(self, budget_hours):
        """Set budget hours.

        Args:
            budget_hours(int): Budget hours.

        """
        self.budget_hours = budget_hours

    def get_budget_hours(self):
        """Get budget_hours.

        Returns:
            int: Budget hours.

        """
        return self.budget_hours

    def set_total_hours(self, total_hours):
        """Set total hours.

        Args:
            total_hours(str): Total hours.

        """
        self.total_hours = total_hours
 
    def get_total_hours(self):
        """Get total hours.

        Returns:
            str: Total hours.

        """
        return self.total_hours

    def set_billed_hours(self, billed_hours):
        """Set billed hours.
 
        Args:
            billed_hours(str): Billed hours.

        """
        self.billed_hours = billed_hours

    def get_billed_hours(self):
        """Get billed hours.

        Args:
            str: billed hours.

        """
        return self.billed_hours

    def set_un_billed_hours(self, un_billed_hours):
        """Set unbilled hours.
       
        Args:
            un_billed_hours(str): Unbilled hours.

        """
        self.un_billed_hours = un_billed_hours

    def get_un_billed_hours(self):
        """Get unbilled hours.

        Returns: 
            str: Unbilled hours.

        """
        return self.un_billed_hours
 
    def set_project_id(self, project_id):
        """Set project_id.

        Args:
            project_id(str): Project id.

        """
        self.project_id = project_id

    def get_project_id(self):
        """Get project id.

        Returns:
            str: Project id.

        """
        return self.project_id

    def set_project_name(self, project_name):
        """Set project name.
 
        Args:
            project_name(str): Project name.

        """
        self.project_name = project_name

    def get_project_name(self):
        """Get project name.

        Returns:
            str: Project name.

        """
        return self.project_name

    def set_currency_id(self, currency_id):
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(str): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id

    def set_customer_name(self, customer_name):
        """Set customer name.

        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns: 
            str: Customer name.

        """
        return self.customer_name

    def set_log_time(self, log_time):
        """Set log time.

        Args:
            log_time(str): Log time.

        """
        self.log_time = log_time

    def get_log_time(self):
        """Get log time.

        Returns:
            str: Log time.

        """
        return self.log_time

    def to_json(self):
        """This method is used to convert task object to jsno format.

        Returns:
            dict: Dictionary containing json object for task.

        """
        data = {}
        if self.task_name != '':
            data['task_name'] = self.task_name
        if self.description != '':
            data['description'] = self.description
        if self.rate > 0:
            data['rate'] = self.rate
        if self.budget_hours > 0:
            data['budget_hours'] = self.budget_hours
        return data



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.CreditNotesParser import CreditNotesParser
from os.path import basename
from json import dumps
from books.api.Api import Api

base_url = Api().base_url + 'creditnotes/'
parser = CreditNotesParser()
zoho_http_client = ZohoHttpClient()

class CreditNotesApi:
    """Creditnotes api class is used to 

    1.List Credits notes with pagination.
    2.Get details of a credit note.
    3.Create a credit note for a custoomer.
    4.Update an existing creditnote.
    5.Delete an existing creditnote.
    6.Change an existing creditnote status to open.
    7.Mark an existing creditnote as void.
    8.Email a creditnote to a customer.
    9.Get email history of a credit note.
    10.Get email content of a credit note.
    11.Update the billing address of an existing credit note.
    12.Update the shipping address of an existing credit note.
    13.Get all credit note pdf templates.
    14.Update the pdf template associated with the creditnote.
    15.List invooices to which the credit note is applied.
    16.Apply credit note to existing invoices.
    17.Delete the credits applied to an invoice.
    18.List all refunds with pagination.
    19.List all refunds of an existing credit note.
    20.Get refund of a particular credit note.
    21.Refund credit note amount.
    22.Update the refunded transaction.
    23.Delete a credit note refund.
    24.Get history and comments of a credit note.
    25.Add a comment to an existing credit note.
    26.Delete a credit note comment.

    """
      
    def __init__(self, authtoken, organization_id):
        """Initialize credit notes api using user's authtoken and organization 
            id.

        Args:
            authtoken(str): User's authtoken.
            organization_id(str): User's organization id.

        """
        self.details = {
            'authtoken':authtoken, 
            'organization_id':organization_id
            }

    def get_credit_notes(self, parameter=None):
        """List all credit notes with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed. Defaults to None.


        Returns:
            instance: Credit notes list object.
        
        Raises: 
            Books Exception: If status is '200' or '201'.
 
        """
        response = zoho_http_client.get(base_url, self.details, parameter)
        return parser.creditnotes_list(response) 

    def get_credit_note(self, creditnote_id, print_pdf=None, accept=None):
        """Get details of a credit note.

        Args:
            creditnote_id(str): Credit note id.
            print_pdf(bool, optional): Export credit note pdf with default 
                option. Allowed values are true, false,on and off.

        Returns:
            instance: Credit note object.

        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + creditnote_id
        if print_pdf is not None and accept is not None:
            query = {
                'print':str(print_pdf).lower(), 
                'accept':accept
                }
            response = zoho_http_client.getfile(url, self.details, query)
            return response
        elif print_pdf is not None:
            query = {
                'print':str(print_pdf).lower()
                } 
            if print_pdf:
                query.update({
                    'accept':'pdf'
                    })
                response = zoho_http_client.getfile(url, self.details, query)
                return response
            else:
                response = zoho_http_client.get(url, self.details, query)
                return parser.get_creditnote(response) 
        elif accept is not None:
            query = {
                'accept':accept
                }
            response = zoho_http_client.getfile(url, self.details, query)
            return response 
        else:
            response = zoho_http_client.get(url, self.details)
            return parser.get_creditnote(response) 
  
    def create(self, credit_note, invoice_id=None, \
        ignore_auto_number_generation=None):
        """Creates a credit note.

        Args:
            credit_note(instance): Credit note object.
            invoice_id(str, optional): Invoice id for which invoice has to be 
                applied.
            ignore_auto_number_generation(bool, optional): True to ignore auto 
                number generation else False. If set True this parameter 
                becomes mandatory.
          
        Returns:
            instance: Credit note object.
 
        """
        json_object = dumps(credit_note.to_json())
        data = {
            'JSONString': json_object
            }
        if invoice_id is not None and ignore_auto_number_generation is not \
            None:
            query = {
                'invoice_id':invoice_id, 
                'ignore_auto_number_generation':ignore_auto_number_generation 
                }
        elif invoice_id is not None:
            query = {
                'invoice_id':invoice_id
                }
        elif ignore_auto_number_generation is not None:
            query = {
                'ignore_auto_number_generation':ignore_auto_number_generation
                }
        else:
            query = None
        response = zoho_http_client.post(base_url, self.details, data, query)
        return parser.get_creditnote(response) 

    def update(self, credit_note_id, credit_note, \
        ignore_auto_number_generation=None):
        """Update an existing credit note.

        Args:
            credit_note_id(str): Credit note id.
            credit_note(instance): Credit note object.
            ignore_auto_number_generation(bool, optional): True to ignore auto 
                number generation. If this is set true then this parameter is 
                mandatory.

        Returns:
            instance: Credit note object.

        """   
        url = base_url + credit_note_id
        json_object = dumps(credit_note.to_json())
        data = {
            'JSONString': json_object
            }
        if ignore_auto_number_generation is not None:
            query = {
                'ignore_auto_number_generation':ignore_auto_number_generation
                } 
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_creditnote(response)

    def delete(self, creditnote_id):
        """Delete an existing credit note.

        Args:
            creditnote_id(str): Credit note id.

        Returns: 
            str: Success message('The credit note has been deleted.').

        """
        url = base_url + creditnote_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

    def convert_to_open(self, creditnote_id):
        """Change an existing credit note status to open.
        Args:
            creditnote_id(str): Credit note id.

        Returns: 
            str: Success message(the status of the credit note has been changed 
                to open.

        """
        url = base_url + creditnote_id  + '/status/open'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 

    def void_credit_note(self, creditnote_id):
        """Mark an existing credit note as void.

        Args:
            creditnote_id(str): Credit note id.

        Returns:
            str: Success message('The credit note has been marked as void.').

        """
        url = base_url + creditnote_id + '/status/void'
        response = zoho_http_client.post(url, self.details, '')
        return parser.get_message(response) 
  
    def email_credit_note(self, creditnote_id, email, attachment=None,
                          customer_id=None):
        """Email a credit note to the customer.

        Args:
            creditnote_id(str): Creditnote id.
            email(instance): Email object.
            attachment(list of dict, optional): List of dictionary containing 
                details of the files to be attached.
            customer_id(str, optional): Id of the customer.

        Returns:
            str: Success message('Your credit note has been sent.').

        """  
        url = base_url + creditnote_id + '/email'
        json_object = dumps(email.to_json()) 
        data = {
            'JSONString': json_object
            }
        if attachment is not None and customer_id is not None:
            query = {
                'customer_id':customer_id
                }
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename':basename(value),         
                        'content':open(value).read()
                        } 
                    }
                file_list.append(attachments)
        elif attachment is not None:
            file_list = []
            for value in attachment:
                attachments = {
                    'attachments': {
                        'filename':basename(value), 
                        'content':open(value).read()
                        }
                    }
                file_list.append(attachments)
            query = None
        elif customer_id is not None:
            query = {
                'customer_id': customer_id
                }
            file_list = None
        else:
            query = None
            file_list = None
        response = zoho_http_client.post(url, self.details, data, query, 
                                         file_list)
        return parser.get_message(response) 

    def email_history(self, creditnote_id):
        """Get email history of a credit note.

        Args:
            creditnote_id(str): Credit note id.
        
        Returns: 
            instance: Email object.

        """
        url = base_url + creditnote_id + '/emailhistory'
        response = zoho_http_client.get(url, self.details)
        return parser.email_history(response) 

    def get_email_content(self, creditnote_id, email_template_id=None):
        """Get email content of a credit note.

        Args:
            creditnote_id(str): Creditnote id.
            email_template_id(str, optional): Email template id.

        Returns:
            instance): Email object.

        """
        url = base_url + creditnote_id + '/email'
        if email_template_id is not None:
            query = {
                'email_template_id': email_template_id
                }
        else:
            query = None
        response = zoho_http_client.get(url, self.details, query)
        return parser.email(response) 
  
    def update_billing_address(self, creditnote_id, address, 
                               is_update_customer=None):
        """Update billing address.

        Args:
            creditnote_id(str): Credit note id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update customer else 
                False.

        Returns:
            instance: Address object.

        """
        url = base_url + creditnote_id + '/address/billing'
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer': is_update_customer
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_billing_address(response)

    def update_shipping_address(self, creditnote_id, address, 
                                is_update_customer=None):
        """Update shipping address.

        Args:
            creditnote_id(str): Credit note id.
            address(instance): Address object.
            is_update_customer(bool, optional): True to update customer 
                else False.
        
        Returns:
            instance: Address object.

        """
        url = base_url + creditnote_id + '/address/shipping'
        json_object = dumps(address.to_json())
        data = {
            'JSONString': json_object
            }
        if is_update_customer is not None:
            query = {
                'is_update_customer': is_update_customer
                }
        else:
            query = None
        response = zoho_http_client.put(url, self.details, data, query)
        return parser.get_shipping_address(response) 

    def list_credit_note_template(self):
        """Get all credit note pdf templates.

        Returns:
            list of instance: List of templates object.

        """
        url = base_url + 'templates'
        response = zoho_http_client.get(url, self.details)
        return parser.list_templates(response) 
 
    def update_credit_note_template(self, creditnote_id, template_id):
        """Update credit note template.

        Args:
            creditnote_id(str): Credit note id.
            template_id(str): Template id.

        Returns:
            str: Success message('The credit note has been updated.').

        """
        url = base_url + creditnote_id + '/templates/' + template_id
        response = zoho_http_client.put(url, self.details, '')
        return parser.get_message(response) 

#### Apply to Invoice------------------------------------------------------------------------------------------------------------

  
    def list_invoices_credited(self, creditnote_id): 
        """List invoices to which credit note is applied.

        Args:
            creditnote_id(str): Credit note id.

        Returns:
            list of instance: List of invoices credited object.
 
        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + creditnote_id + '/invoices'  
        response = zoho_http_client.get(url, self.details)
        return parser.list_invoices_credited(response) 
  
    def credit_to_invoice(self, creditnote_id, invoice):
        """List invoices to which the credited note is applied.

        Args:
            creditnote_id(str): Credit note id.
            invoice(instance): Invoice object.

        Returns:
            list of invoices: List of invoice objects.

        """
        url = base_url + creditnote_id + '/invoices'
        invoices = {
            'invoices': []
            }
        for value in invoice:
            data = {}
            data['invoice_id'] = value.get_invoice_id()
            data['amount_applied'] = value.get_amount_applied()
            invoices['invoices'].append(data)
        json_string = {
            'JSONString': dumps(invoices)
            }
        response = zoho_http_client.post(url, self.details, json_string)
        return parser.credit_to_invoice(response)
      
    def delete_invoices_credited(self, creditnote_id, creditnote_invoice_id):
        """Delete the credits applied to an invoice.

        Args:
            creditnote_id(str): Credit note id.
            creditnote_invoice_id(str): Credit note invoice id.

        Returns:
            str: Success message('Credits applied to an invoice have been 
                 deleted.').

        """
        url = base_url + creditnote_id + '/invoices/' + creditnote_invoice_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 

##  REFUND-----------------------------------------------------------------------------------------------------------------------

    def list_credit_note_refunds(self, customer_id=None, sort_column=None):
        """List all refunds wiht pagination.

        Args:
            customer_id(str, optional): Customer id.
            sort_column(str, optional): Sort refund list. Allowed values are 
                refund_mode, reference_number, date, creditnote_number, 
                customer_name, amount_bcy and amount_fcy.

        Returns:
            instance: Credit notes refund list object.

        """       
        url = base_url + 'refunds'
        if customer_id is not None and sort_column is not None:
            parameter = {
                'customer_id':customer_id, 
                'sort_column':sort_column
                }
        elif sort_column is not None or customer_id is not None:
            parameter = {
                'sort_column': sort_column
                } if sort_column is not None else {
                'customer_id': customer_id
                }
        else:
            parameter = None
        response = zoho_http_client.get(url, self.details, parameter)
        return parser.creditnote_refunds(response) 

    def list_refunds_of_credit_note(self, creditnote_id): 
        """List all refunds of an existing credit note.

        Args:
            creditnote_id(str): Credit note id.

        Returns:
            instance: Creditnotes refund list.

        """
        url = base_url + creditnote_id + '/refunds'
        response = zoho_http_client.get(url, self.details)
        return parser.creditnote_refunds(response)

    def get_credit_note_refund(self, creditnote_id, creditnote_refund_id):
        """Get credit note refund.

        Args:
            creditnote_id(str): Credit note id.
            creditnote_refund_id(str): Creditnote refund id.

        Returns:
            instance: Creditnote refund object.

        """       
        url = base_url + creditnote_id + '/refunds/' + creditnote_refund_id
        response = zoho_http_client.get(url, self.details)
        return parser.get_creditnote_refund(response)
  
    def refund_credit_note(self, creditnote_id, creditnote):
        """Refund credit note amount.

        Args:
            creditnote_id(str): Credit note id.

        Returns:
            creditnote(instance): Credit note object.

        """
        url = base_url + creditnote_id + '/refunds'
        json_object = dumps(creditnote.to_json())
        data = { 
            'JSONString': json_object
            }
        response = zoho_http_client.post(url, self.details, data)
        return parser.get_creditnote_refund(response)
     
    def update_credit_note_refund(self, creditnote_id, creditnote_refund_id, 
                                  creditnote):
        """Update the refunded transaction.

        Args:
            creditnote_id(str): Credit note id.
            creditnote_refund_id(str): Credit note refund id.
            creditnote(instance): Credit note object.

        Returns:
            instance: Creditnote object.

        """
        url = base_url + creditnote_id + '/refunds/' + creditnote_refund_id
        json_object = dumps(creditnote.to_json())
        data = { 
            'JSONString': json_object
            }
        response = zoho_http_client.put(url, self.details, data)
        return parser.get_creditnote_refund(response) 

    def delete_credit_note_refund(self, creditnote_id, creditnote_refund_id): 
        """Delete a credit note refund.

        Args:
            creditnote_id(str): Credit note id.
            creditnote_refund_id(str): Credit note refund id.
  
        Returns:
            str: Success message('The refund has been successfully deleted.').

        """
        url = base_url + creditnote_id + '/refunds/' + creditnote_refund_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response)

## Comments and History----------------------------------------------------------------------------------------------------------


    def list_creditnote_comments_history(self, creditnote_id):
        """Get history and comments of a credit note.

        Args:
            creditnote_id(str): Credit note id.
 
        Returns:
            instance: Comments list object.

        """
        url = base_url + creditnote_id + '/comments'
        response = zoho_http_client.get(url, self.details)
        return parser.comments_list(response) 
  
    def add_comment(self, creditnote_id, comments):
        """Add a comment to an existing credit note.

        Args:
            creditnote_id(str): Credit note id.
            comments(instance): Comments object.

        Returns:
            instance: Comments object.

        """
        url = base_url + creditnote_id + '/comments'
        data = {}
        data['description'] = comments.get_description()
        json_string = {
            'JSONString': dumps(data)
            }
        response = zoho_http_client.post(url, self.details, json_string)
        return parser.get_comment(response) 

    def delete_comment(self, creditnote_id, comment_id):
        """Delete a credit note comment.

        Args:
            creditnote_id(str): Credit note id.
            comment_is(str): Comment id.

        Returns:
            str: Success message('The comment has been deleted.').

        """
        url = base_url + creditnote_id + '/comments/' + comment_id
        response = zoho_http_client.delete(url, self.details)
        return parser.get_message(response) 




#$Id$

from books.model.Address import Address
from books.model.DefaultTemplate import DefaultTemplate

class Contact:
    """This class is used to create object for contacts."""
    def __init__(self):
        """Initialize the parameters for contacts object."""
        self.contact_id = ''
        self.contact_name = ''
        self.company_name = ''
        self.contact_salutation = ''
        self.has_transaction = None
        self.contact_type = ''
        self.is_crm_customer = None
        self.primary_contact_id = ''
        self.price_precision = 0
        self.payment_terms = 0
        self.payment_terms_label = ''
        self.currency_id = ''
        self.currency_code = ''
        self.currency_symbol = ''
        self.outstanding_receivable_amount = 0.00
        self.outstanding_receivable_amount_bcy = 0.00
        self.outstanding_payable_amount = 0.00
        self.outstanding_payable_amount_bcy = 0.00
        self.unused_credits_receivable_amount = 0.00
        self.unused_credits_receivable_amount_bcy = 0.00
        self.unused_credits_payable_amount = 0.00
        self.unused_credits_payable_amount_bcy = 0.00
        self.status = ''
        self.payment_reminder_enabled = None
        self.notes = ''
        self.created_time = ''
        self.last_modified_time = ''
        self.first_name = ''
        self.last_name = ''
        self.email = ''
        self.phone = ''
        self.mobile = ''
        self.custom_fields = []
        self.track_1099 =  None   
        self.tax_id_type =  ''
        self.tax_id_value =  ''
        self.billing_address = Address()
        self.shipping_address = Address()
        self.contact_persons = []
        self.default_templates = DefaultTemplate()
        self.gst_treatment = ''
        self.place_of_contact = ''
        self.gst_no = ''
    
    def set_gst_no(self, gst_no):
        """Set gst_no for the contact.
 
        Args: 
            gst_no(str): gst_no of the contact.
        
        """
        self.gst_no = gst_no
 
    def get_gst_no(self):
        """Get gst_no of the contact.
       
        Returns:
            str: gst_no of the contact.
   
        """
        return self.gst_no

    def set_place_of_contact(self, place_of_contact):
        """Set place_of_contact for the contact.

        Args:
            place_of_contact(str): place_of_contact of the contact.

        """
        self.place_of_contact = place_of_contact

    def get_place_of_contact(self):
        """Get place_of_contact of the contact.

        Returns:
            str: place_of_contact of the contact.

        """
        return self.place_of_contact

    def set_gst_treatment(self, gst_treatment):
        """Set gst_treatment for the contact.

        Args:
            gst_treatment(str): gst_treatment of the contact.

        """
        self.gst_treatment = gst_treatment

    def get_gst_treatment(self):
        """Get gst_treatment of the contact.

        Returns:
            str: gst_treatment of the contact.

        """
        return self.gst_treatment

    def set_contact_id(self, contact_id):
        """Set contact id for the contact.

        Args:
            contact_id(str): Contact id of the contact.

        """
        self.contact_id = contact_id

    def get_contact_id(self):
        """Get contact_id of the contact.

        Returns:
            str: Contact id of the contact.

        """
        return self.contact_id

    def set_contact_name(self, contact_name):
        """Set contact name for the contact.
        
        Args:
            contact_name(str): Contact name of the contact.
         
        """
        self.contact_name = contact_name

    def get_contact_name(self):
        """Get contact name of the contact.
          
        Returns:
            str: Contact name of the contact.
      
        """ 
        return self.contact_name
      
    def set_has_transaction(self, has_transaction):
        """Set whether the contact has transaction or not.
        
        Args:
            has_transaction(bool): True if the contact has transactions 
                else False.
         
        """
        self.has_transaction = has_transaction
 
    def get_has_transaction(self):
        """Get whether the contact has transaction or not.
        
        Returns:
            bool: True if the contact has transactions else False.
      
        """
        return self.has_transaction

    def set_contact_type(self, contact_type):
        """Set contact type of the contact.
      
        Args:
            contact_type(str): Contact type of the contact.
  
        """
        self.contact_type = contact_type
 
    def get_contact_type(self):
        """Get contact type of the contact.
        
        Returns:
            str: Contact type of the contact.
          
        """
        return self.contact_type

    def set_is_crm_customer(self, is_crm_customer):
        """Set whether the contact is crm customer or not.
     
        Args:
            is_crm_customer(bool): True if the contact is crm customer else 
                False.
        
        """
        self.is_crm_customer = is_crm_customer

    def get_is_crm_customer(self):
        """Get whether the contact is crm customer or not.
     
        Returns:
            bool: True if the contact is crm customer else False .
        
        """
        return self.is_crm_customer

    def set_primary_contact_id(self, primary_contact_id):
        """Set primary contact id for the contact.
       
        Args:
            primary_contact_id(str): Primary contact id for the contact.
         
        """
        self.primary_conatact_id = primary_contact_id
 
    def get_primary_conatact_id(self):
        """Get primary contact id for the contact.
 
        Returns:
            str: Primary contact id for the contact.

        """ 
        return self.primary_conatact_id

    def set_payment_terms(self, payment_terms):
        """Set payment terms for the contact.
 
        Args:
            payment_terms(int): Payment terms for the contact.
 
        """
        self.payment_terms = payment_terms

    def get_payment_terms(self):
        """Get payment terms of the contact.
         
        Returns:
            int: Payment terms of the contact.
 
        """
        return self.payment_terms

    def set_payment_terms_label(self, payment_terms_label):
        """Set payment terms label for the contact.
     
        Args:
            payment_terms_label(str): Payment terms for the contact.
  
        """
        self.payment_terms_label = payment_terms_label

    def get_payment_terms_label(self):
        """Get payment terms label of the contact.
 
        Returns:
            str: Payment terms label of the contact.
 
        """
        return self.payment_terms_label

    def set_currency_id(self, currency_id):
        """ Set currency id for the contact.
        
        Args:
            currency_id(str): Currency id for the contact.
        
        """
        self.currency_id = currency_id
 
    def get_currency_id(self):
        """Get currency id of the contact.

        Args:
            currency_id(str): Currency id for the contact.
 
        """
        return self.currency_id

    def set_currency_code(self, currency_code):
        """Set currency code for the contact.
         
        Args: 
            currency_code(str): Currency code for the contact.
 
        """
        self.currency_code = currency_code 
 
    def get_currency_code(self):
        """Get currency code of the contact.

        Returns:
            str: Currency code of the contact.
 
        """
        return self.currency_code

    def set_currency_symbol(self, currency_symbol):
        """Set currency symbol for the contact.
        
        Args:
            currency_symbol(str): Currency symbol for the contact.
 
        """
        self.currency_symbol = currency_symbol

    def get_currency_symbol(self):
        """Get currency symbol of the contact.
      
        Returns:
            str: Currency symbol of the contact.
 
        """
        return self.currency_symbol

    def set_outstanding_receivable_amount(self, outstanding_receivable_amount):
        """Set outstanding receivable amount for the contact.

        Args: 
            outstanding_receivable_amount(float): Outstanding receivable amount
                 for the contact.
     
        """
        self.outstanding_receivable_amount = outstanding_receivable_amount

    def get_outstanding_receivable_amount(self):
        """Get outstanding receivable amount of the contact.
       
        Returns:
            float: Outstanding receivable amount of the contact.

        """
        return self.outstanding_receivable_amount 

    def set_outstanding_receivable_amount_bcy(self, \
        outstanding_receivable_amount_bcy):
        """Set outstanding receivable amount bcy for the contact.
 
        Args:
            outstanding_receivable_amount_bcy(float): Outstanding receivable 
                amount bcy for the contact.
   
        """
        self.outstanding_receivable_amount_bcy = \
        outstanding_receivable_amount_bcy

    def get_outstanding_receivable_amount_bcy(self):
        """Get the outstanding receivable amount bcy of the contact.
        
        Returns:
            float: Outstanding receivable amount bcy of the contact.

        """
        return self.outstanding_receivable_amount_bcy

    def set_outstanding_payable_amount(self, outstanding_payable_amount):
        """Set the outstanding payable amount for the contact.
        
        Args:
            outstanding_payable_amount(float): Outstanding payable amount for 
                the contact.

        """
        self.outstanding_payable_amount = outstanding_payable_amount
 
    def get_outstanding_payable_amount(self):
        """Get the outstanding payable amount of the contact.
      
        Returns:
            float: Outstanding payable amount of the contact.
         
        """
        return self.outstanding_payable_amount

    def set_outstanding_payable_amount_bcy(self, \
        outstanding_payable_amount_bcy):
        """Set outstanding payable amount bcy for the contact.
       
        Args:
            outstanding_payable_amount_bcy(float): Outstanding payable amount 
                bcy for the contact.
  
        """
        self.outstanding_payable_amount_bcy = outstanding_payable_amount_bcy

    def get_outstanding_payable_amount_bcy(self):
        """Get outstanding payable amount bcy of the contact.
         
        Returns:
            float: Outstanding payable amount bcy of the contact.

        """
        return self.outstanding_payable_amount_bcy

    def set_unused_credits_receivable_amount(self, \
        unused_credits_receivable_amount):
        """Set unused credits receivable amount for the contact.
 
        Args: 
            unused_credits_receivable_amount(float): Unused credits receivable 
                amount for the contact.
       
        """
        self.unused_credits_receivable_amount = \
        unused_credits_receivable_amount

    def get_unused_credits_receivable_amount(self):
        """Get unused credits receivable amount of the contact.
 
        Returns:
            float: Unused credits receivable amount for the contact.
         
        """
        return self.unused_credits_receivable_amount

    def set_unused_credits_receivable_amount_bcy(self, \
        unused_credits_receivable_amount_bcy):
        """Set unused credits receivable amount bcy for the contact.
        
        Args:
            unused_credits_receivable_amount_bcy(float): Unused credits 
                receivable amount bcy for the contact.
 
        """
        self.unused_credits_receivable_amount_bcy = \
        unused_credits_receivable_amount_bcy

    def get_unused_credits_receivable_amount_byc(self):
        """Get unused credits receivable amount bcy of the contact.
        
        Returns:
            float: Unused credits receivable amount bcy of the contact.
    
        """
        return self.unused_credits_receivable_amount_bcy

    def set_unused_credits_payable_amount(self, unused_credits_payable_amount):
        """Set unused credits payable amount for the contact.
        
        Args:
            unused_credits_payable_amount(float): Unused credits payable 
                amount for the contact.

        """
        self.unused_credits_payable_amount = unused_credits_payable_amount

    def get_unused_credits_payable_amount(self):
        """Get unused payable amount of the contact.
         
        Returns:
            float: Unused payable amount of the contact.

        """
        return self.unused_credits_payable_amount

    def set_unused_credits_payable_amount_bcy(self, \
        unused_credits_payable_amount_bcy):
        """Set unused credits payable amount bcy for the contact.
        
        Args:
            unused_credits_payable_amount_bcy(float): Unused credits payable 
                amount bcy for the contact.
 
        """
        self.unused_credits_payable_amount_bcy = \
        unused_credits_payable_amount_bcy

    def get_unused_credits_payable_amount_bcy(self):
        """Get unused credits payable amount bcy of the contact.
 
        Returns:
            float: Unused credits payable amount bcy of the contact.

        """
        return self.unused_credits_payable_amount_bcy

    def set_status(self, status):
        """Set status for the contact.
       
        Args:
            status(str): Status of the contact.
        
        """
        self.status = status
 
    def get_status(self):
        """Get status of the contact.
        
        Returns:
            str: Status of the contact.
         
        """
        return self.status

    def set_payment_reminder_enabled(self, payment_reminder_enabled):
        """Set whether to enabe payment reminder for the contact.
       
        Args: 
            payment_reminder_enabled(bool): True if enable payment reminder 
                else false.

        """
        self.payment_reminder_enabled = payment_reminder_enabled
  
    def get_payment_reminder_enabled(self):
        """Get whether the payment reminder is enabled or not
 
        Returns:
            bool: True if payment reminder is enabled else false.

        """
        return self.payment_reminder_enabled   

    def set_notes(self, notes):
        """Set notes for the contact.
   
        Args:
            notes(str): Notes for contact.
   
        """
        self.notes = notes
 
    def get_notes(self):
        """Get notes of the contact.
 
        Returns:
            str: Notes of the contact.
  
        """
        return self.notes

    def set_created_time(self, created_time):
        """Set created time for the contact.
        
        Args:
            created_time(str): Created time for the contact.
 
        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created of the contact.
       
        Returns:
            str: Created time of the contact.
 
        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time for the contact.
        
        Args:
            last_modified_time(str): Last modified time for the contact.

        """
        self.last_modified_time = last_modified_time  

    def get_last_modified_time(self):
        """Get last modified time of the contact.
        
        Returns:
            str: Last modified time of the contact.
      
        """
        return self.last_modified_time
 
    def set_billing_address(self, billing_address):
        """Set billing address for the contact.
        
        Args:
            billing_address(Address): Billing address for the contact.
    
        """
        self.billing_address = billing_address
  
    def get_billing_address(self):
        """Get billing address of the contact.
         
        Returns:
            Address: Billing address of the contact.

        """
        return self.billing_address

    def set_shipping_address(self, shipping_address):
        """Set shipping address for the contact.
        
        Args:
            shipping_address(Address): Shipping address for the contact.

        """
        self.shipping_address = shipping_address

    def get_shipping_address(self):
        """Get shipping address of the contact.
         
        Returns:
            Address: Shipping address of the contact.

        """
        return self.shipping_address
   
    def set_contact_persons(self, contact_person):
        """Set contact persons for the contact.
        
        Args:
            contact_person(list): List of contact persons object.
        
        """
        self.contact_persons.extend(contact_person)
 
    def get_contact_persons(self):
        """Get contact persons of a contact.
        
        Returns:
            list: List of contact persons.

        """
        return self.contact_persons

    def set_default_templates(self, default_templates):
        """Set default templates for the contact.
        
        Args:
            default_templates(instance): Default templates object.
        
        """
        self.default_templates = default_templates
 
    def get_default_templates(self):
        """Get default templates of the contact.
        
        Returns:
            instance: Default templates instance.
  
        """
        return self.default_templates
  

    def set_custom_fields(self, custom_field):
        """Set custom fields for a contact.
       
        Args: 
            custom_field(instance): Custom field object.
  
        """
        self.custom_fields.append(custom_field)

    def get_custom_fields(self):
        """Get custom fields of the contact.
        
        Returns:
            instance: Custom field of the contact.
 
        """
        return self.custom_fields

    def set_company_name(self, company_name):
        """Set company name for the contact.
        
        Args:
            company_name(str): Company name of the contact.

        """
        self.company_name = company_name
 
    def get_company_name(self):
        """Get company name of the contact.
        
        Returns:
            str: cCompany name of the contact.
        """
        return self.company_name

    def set_contact_salutation(self, contact_salutation):
        """Set salutation for the contact.
        
        Args: 
            contact_salutation(str): Salutation of the contact.
       
        """
        self.contact_salutation = contact_salutation
 
    def get_contact_salutation(self):
        """Get salutation of the contact.
      
        Returns:
            str: Salutation of the contact
  
        """
        return self.contact_salutation

    def set_price_precision(self, price_precision):
        """Set price precision for the contact.
        
        Args:
            price_precision(int): Price precision for the contact.
       
        """
        self.price_precision = price_precision
 
    def get_price_precision(self):
        """Get price precision of the contact.
 
        Returns:
            int: Price precision of the contact.
 
        """
        return self.price_precision

    def set_track_1099(self, track_1099):
        """Set to track a contact for 1099 reporting.
        
        Args:
            track_1099(bool): True to track a contact for 1099 reporting else 
                False.
 
        """
        self.track_1099 = track_1099
 
    def get_track_1099(self):
        """Get whether a contact is set for 1099 tracking.
          
        Returns:
            bool: True if a contact is set for 1099 tracking else False.
 
        """
        return self.track_1099 

    def set_tax_id_type(self, tax_id_type):
        """Set tax id type for a contact.
      
        Args: 
            tax_id_type(str): tax id type for a contact
 
        """
        self.tax_id_type = tax_id_type

    def get_tax_id_type(self):
        """Get tax id type of a contact.
        
        Returns:
            str: Tax id type for a contact.
    
        """
        return self.tax_id_type

    def set_tax_id_value(self, tax_id_value):
        """Set tax id value for a contact.
        
        Args:
            tax_id_value(str): Tax id value for a contact.
 
        """
        self.tax_id_value = tax_id_value

    def get_tax_id_value(self):
        """Get tax id value of a contact.
        
        Returns:
            str: Tax id value of a contact.

        """
        return self.tax_id_value

    def set_first_name(self, first_name): 
        """Set first name.
 
        Args:
            first_name = First name.    
     
        """
        self.first_name = first_name
       
    def get_first_name(self):
        """Get first name.
 
        Returns:
            str: First name.
 
        """
        return self.first_name

    def set_last_name(self, last_name):
        """Set last name.

        Args:
            last_name = Last name.
       
        """
        self.last_name = last_name

    def get_last_name(self):
        """Get last name.

        Returns:
            str: Last name.
  
        """
        return self.last_name

    def set_email(self, email):
        """Set email.
 
        Args:
            email(str): Email.

        """
        self.email = email

    def get_email(self):
        """Get email.

        Returns:
            str: Email.
 
        """
        return self.email

    def set_phone(self, phone):
        """Set phone.
     
        Args: 
            phone(str): Phone.

        """
        self.phone = phone

    def get_phone(self):
        """Get phone.

        Returns:
            str: Phone.

        """
        return self.phone

    def set_mobile(self, mobile):
        """Set mobile.
 
        Args:
            mobile(str): Mobile.

        """
        self.mobile = mobile

    def to_json(self):
        """This method is used to convert the contact object to JSON object.

        Returns:
            dict: Dictionary containing details of contact object.

        """
        data = {}
        data['email'] = self.email
        if self.company_name != '':
            data['company_name'] = self.company_name
        if self.contact_name != '':
            data['contact_name'] = self.contact_name
        if self.payment_terms != '':
            data['payment_terms'] = self.payment_terms
        if self.payment_terms_label != '':
            data['payment_terms_label'] = self.payment_terms_label
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.billing_address is not None: 
            billing_address = self.billing_address
            data['billing_address'] = billing_address.to_json()
        if self.shipping_address is not None:
            shipping_address = self.shipping_address
            data['shipping_address'] = shipping_address.to_json()
        if self.gst_no != '':
            data['gst_no'] = self.gst_no
        if self.place_of_contact != '':
            data['place_of_contact'] = self.place_of_contact
        if self.gst_treatment != '':
            data['gst_treatment'] = self.gst_treatment
        if self.contact_persons:
            data['contact_persons'] = []
            for value in self.contact_persons:
                data['contact_persons'].append(value.to_json())
        if self.notes != '':
            data['notes'] = self.notes
        return data



#$Id$#

from books.util.ZohoHttpClient import ZohoHttpClient
from books.parser.BillsParser import BillsParser
from books.api.Api import Api
from os.path import basename
from json import dumps

base_url = Api().base_url + 'bills/'
zoho_http_client = ZohoHttpClient()
parser = BillsParser()

class BillsApi:
    """Bills Api is used to

    1.List all bills with pagination
    2.Get the details of a bill.
    3.Create a bill received from a vendor.
    4.Update an existing bill.
    5.delete an existing bill.
    6.Mark a bill status as void.
    7.Mark a void bill as open.
    8.Update the billing address.
    9.Get the list of payments made for the bill.
    10.Apply the vendor credits from excess vendor payments to a bill. 
    11.Delete a payment made to a bill.
    12.Returns a file attached to the bill.
    13.Attach a file to a bill.
    14.Delete the file attached to a bill.
    15.Get the cmplete history and comments of a bill.
    16.Add a comment for a bill.
    17.Delete a bill comment.

    """
    def __init__(self, authtoken, organization_id):
        """Initialize Bills api using user's authtoken and organization id.

        Args: 
            authotoken(str): User's Authtoken.
            organization id(str): User's Organization id.

        """
        self.details = {
            'authtoken': authtoken, 
            'organization_id': organization_id
            }

    def get_bills(self, parameter=None):
        """List all bills with pagination.

        Args:
            parameter(dict, optional): Filter with which the list has to be 
                displayed.

        Returns:
            instance: Bill list object.
 
        """
        resp = zoho_http_client.get(base_url, self.details, parameter) 
        return parser.get_list(resp) 

    def get(self, bill_id):
        """Get the details of a bill.

        Args:
            bill_id(str): Bill id.

        Returns:
            instance: Bill object.

        """
        url = base_url + bill_id
        resp = zoho_http_client.get(url, self.details)
        return parser.get_bill(resp) 

    def create(self, bill, attachment=None):
        """Create a bill received from the vendor.

        Args:
            bill(instance): Bill object.
            attachment(file, optional): File to be attached with the bill. 
                Allowed extensions are gif, png, jpeg, jpg, bmp and pdf.

        Returns:
            instance: Bill object.
 
        """        
        json_object = dumps(bill.to_json())
        data = {
            'JSONString': json_object
            }
        if attachment is None:
            attachments = None
        else: 
            attachments = [{
                'attachment': {
                    'filename': basename(attachment), 
                    'content': open(attachment).read()
                    } 
                }]
        resp = zoho_http_client.post(base_url, self.details, data, None, \
                                     attachments)
        return parser.get_bill(resp) 

    def update(self, bill_id, bill, attachment=None):
        """Update an existing bill.

        Args:
            bill_id(str): Bill id.
            bill(instance): Bill object.
            attachment(file, optional): File to be attached with the bill. 
                Allowed extensions are gif, png, jpeg, jpg, bmp and pdf.
 
        Returns:
            instance: Bill object.
 
        """
        url = base_url + bill_id
        json_object = dumps(bill.to_json())
        data = {
            'JSONString': json_object
            }
        if attachment is None:
            attachments = None
        else: 
            attachments = [{
                'attachment': {
                    'filename': basename(attachment), 
                    'content': open(attachment).read()
                    } 
                }]
        resp = zoho_http_client.put(url, self.details, data, None, \
                                     attachments)
        return parser.get_bill(resp) 

    def delete(self, bill_id):
        """Delete an existing bill.

        Args:
            bill_id(str): Bill id.

        Returns: 
            str: Success message('The bill has been deleted.').
 
        """
        url = base_url + bill_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def void_a_bill(self, bill_id):
        """Mark a bill status as void.
 
        Args:
            bill_id(str): Bill id.

        Returns:
            str: Success message('The bill has been marked as void.').

        """
        url = base_url + bill_id + '/status/void'
        resp = zoho_http_client.post(url, self.details, '') 
        return parser.get_message(resp) 
  
    def mark_a_bill_as_open(self, bill_id):
        """Mark a void bill as open.
 
        Args: 
            bill_id(str): Bill id.

        Returns:
            str: Success message('The status of the bill has been changed from 
                 void to open.').

        """
        url = base_url + bill_id + '/status/open'
        resp = zoho_http_client.post(url, self.details, '')
        return parser.get_message(resp) 
  
    def update_billing_address(self, bill_id, billing_address):
        """Update the billing address for the bill.

        Args:
            bill_id(str): Bill id.
            billing_address(instance): Billing address object.
 
        Returns:
            str: Success message('Billing address update.').

        """
        url = base_url + bill_id + '/address/billing'
        json_object = dumps(billing_address.to_json())
        data = { 
            'JSONString': json_object
            }
        resp = zoho_http_client.put(url, self.details, data)
        return parser.get_message(resp) 
 
    def list_bill_payments(self, bill_id):
        """Get the list of payments made for the bill.

        Args:
            bill_id(str): Bill id.

        Returns:
            instance: Payments list object.

        """
        url = base_url + bill_id + '/payments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_payments_list(resp)
  
    def apply_credits(self, bill_id, bill_payments):
        """Apply the vendor credits from excess vendor payments to a bill.

        Args: 
            bill_id(str): Bill id.
            bill_payments(list of instance): list of payments object.

        Returns:
            str: Success message('Credits have been applied to the bill.').
 
        """ 
        url = base_url + bill_id + '/credits'
        data = {}
        bill_payments_list = []
        for value in bill_payments:
            bill_payment = {}
            bill_payment['payment_id'] = value.get_payment_id()
            bill_payment['amount_applied'] = value.get_amount_applied()
            bill_payments_list.append(bill_payment)

        data['bill_payments'] = bill_payments_list
        json_string = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, json_string)
        return parser.get_message(resp) 

    def delete_a_payment(self, bill_id, bill_payment_id):
        """Delete a payment made to a bill.
 
        Args:
            bill_id(str): Bill id.
            bill_payment_id(str): Bill payment id.

        Returns:
            str: Success message('The payment has been deleted.').
 
        Raises:
            Books Exception: If status is not '200' or '201'.

        """
        url = base_url + bill_id + '/payments/' + bill_payment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 

    def get_a_bill_attachment(self, bill_id, preview=None):
        """Get the file attached to the bill.
 
        Args:
            bill_id(str): Bill id.
            preview(bool, optional): True to get the thumbnail of the 
                attachment else False.
 
        Returns:
            file: File attached to the bill.
 
        """
        if preview is not None: 
            query = {
                'preview': preview
                }
        else:
            query = None
        url = base_url + bill_id + '/attachment'
        resp = zoho_http_client.getfile(url, self.details, query) 
        return resp
 
    def add_attachments_to_a_bill(self, bill_id, attachment):
        """Attach a file to a bill.

        Args:
            bill_id(str): Bill id.
            attachment(file): File to attach. Allowed extensions are gif, png, 
                jpeg, jpg, bmp, pdf, xls, xlsx, doc, docx.

        Returns:
            str: Success message('The document has been deleted.').

        """
        url = base_url + bill_id + '/attachment'
        attachments = [{
                'attachment': {
                    'filename': basename(attachment), 
                    'content': open(attachment).read()
                    } 
                }] 
        data = {
            'JSONString': ''
            }
        resp = zoho_http_client.post(url, self.details, data, None, attachments)
        return parser.get_message(resp) 

    def delete_an_attachment(self, bill_id):
        """Delete the file attached to a bill.
        
        Args:
            bill_id(str): Bill id.
 
        Returns:
            str: Success message('The attachment has been deleted.').
  
        """
        url = base_url + bill_id + '/attachment'
        resp = zoho_http_client.delete(url, self.details) 
        return parser.get_message(resp) 

    def list_bill_comments_and_history(self, bill_id):
        """Get the complete history and comments of a bill.
  
        Args:
            bill_id(str): Bill id.

        Returns: 
            instance: Comments list object.

        """
        url = base_url + bill_id + '/comments'
        resp = zoho_http_client.get(url, self.details)
        return parser.get_comments(resp) 

    def add_comment(self, bill_id, description):
        """Add a comment for a bill.

        Args:
            bill_id(str): Bill id.
            description(str): Description.
 
        Returns:
            str: Success message('Comments added.').

        """
        url = base_url + bill_id + '/comments'
        data = {
            'description': description 
            }
        json_string = {
            'JSONString': dumps(data)
            }
        resp = zoho_http_client.post(url, self.details, json_string)
        return parser.get_comment(resp) 

    def delete_a_comment(self, bill_id, comment_id):
        """Delete a bill comment.

        Args:
            bill_id(str): Bill id.
            comment_id(str): Comment id.
 
        Returns: 
            str: Success message('The comment has been deleted.').
 
        Raises: 
            Books Exception: If status is '200' or '201'.

        """
        url = base_url + bill_id + '/comments/' + comment_id
        resp = zoho_http_client.delete(url, self.details)
        return parser.get_message(resp) 
  



#$Id$

class Expense():
    """This class is used to create objecct for expenses."""
    def __init__(self):
        """Initialize parameters for Expenses object.""" 
        self.account_id = ''
        self.paid_through_account_id = ''
        self.date = ''
        self.amount = 0.0
        self.tax_id = ''
        self.is_inclusive_tax = None
        self.is_billable = None
        self.customer_id = ''
        self.vendor_id = ''
        self.currency_id = ''
        self.exchange_rate = 0.0
        self.project_id = ''
        self.expense_id = ''
        self.expense_item_id = ''
        self.account_name = ''
        self.paid_through_account_name = ''
        self.vendor_name = ''
        self.tax_name = ''
        self.tax_percentage = 0.0
        self.currency_code = ''
        self.tax_amount = 0.0
        self.sub_total = 0.0
        self.total = 0.0
        self.bcy_total = 0.0
        self.reference_number = ''
        self.description = ''
        self.customer_name = ''
        self.expense_receipt_name = ''
        self.created_time = ''
        self.last_modified_time = '' 
        self.status = ''
        self.invoice_id = ''
        self.invoice_number = ''
        self.project_name = ''
        self.recurring_expense_id = ''
        self.product_type = ''
        self.hsn_or_sac = 0
        self.gst_no = ''
        self.gst_treatment = ''
        self.destination_of_supply = ''

    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self): 
        """Get account id.

        Returns:
            str: Account id.

        """
        return self.account_id
  
    def set_paid_through_account_id(self, paid_through_account_id):
        """Set paid through account id.

        Args:
            paid_through_account_id(str): Paid through account id.

        """
        self.paid_through_account_id = paid_through_account_id

    def get_paid_through_account_id(self):
        """Get paid through account id.
 
        Returns:
            str: Paid through account id.

        """
        return self.paid_through_account_id
   
    def set_date(self, date):
        """Set date.

        Args:
            date(str): Date.

        """
        self.date = date

    def get_date(self):
        """Get date.

        Returns:
            str: Date.

        """
        return self.date
  
    def set_amount(self, amount):
        """Set amount.

        Args: 
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount
  
    def set_tax_id(self, tax_id):
        """Set tax id.

        Args:
            tax_id(str): Tax id.

        """
        self.tax_id = tax_id

    def get_tax_id(self):
        """Get tax id.

        Returns:
            str: Tax id.

        """
        return self.tax_id
   
    def set_is_inclusive_tax(self, is_inclusive_tax):
        """Set whether tax is inclusive.
 
        Args:
            is_inclusive_tax(bool): True if tax is inclusive else False.

        """
        self.is_inclusive_tax = is_inclusive_tax

    def get_is_inclusive_tax(self):
        """Get whether tax is inclusive.

        Returns:
            bool: True if tax is inclusive else False.

        """
        return self.is_inclusive_tax
 
    def set_is_billable(self, is_billable):
        """Set whether expenses are billable.

        Args:
            is_billable(bool): True if billable else False.

        """
        self.is_billable = is_billable

    def get_is_billable(self):
        """Get whether expenses are billable.

        Returns:
            bool: True if billable else False.

        """
        return self.is_billable

    def set_customer_id(self, customer_id):
        """Set customer id.

        Args:
            customer_id(str): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self):
        """Get customer id.
 
        Returns:
            str: Customer id.

        """
        return self.customer_id

    def set_vendor_id(self, vendor_id):
        """Set vendor id.
 
        Args:
            vendor_id(str): Vendor id.

        """
        self.vendor_id = vendor_id
 
    def get_vendor_id(self):
        """Get vendor id.

        Returns:
            str: Vendor id.

        """
        return self.vendor_id
    
    def set_currency_id(self, currency_id):
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id

    def get_currency_id(self):
        """Get currency idd.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """
        self.exchange_rate = exchange_rate

    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchangge rate.

        """
        return self.exchange_rate
    
    def set_project_id(self, project_id):
        """Set project id.

        Args:
            project_id(str): Project id.

        """
        self.project_id = project_id
 
    def get_project_id(self):
        """Get project id.

        Returns:
            str: Project id.
 
        """
        return self.project_id

    def set_expense_id(self, expense_id): 
        """Set expense id.

        Args:
            expense_id(str): Expense id.

        """
        self.expense_id = expense_id

    def get_expense_id(self):
        """Get expense id.

        Returns:
            str: Expense id.

        """
        return self.expense_id
 
    def set_expense_item_id(self, expense_item_id):
        """Set expense item id.

        Args:
            expense_item_id(str): Expense item id.

        """
        self.expense_item_id = expense_item_id

    def get_expense_item_id(self):
        """Get expense item id.

        Returns:
            str: Expense item id.

        """
        return self.expense_item_id

    def set_account_name(self, account_name):
        """Set account name.

        Args:
            account_name(str): Account name.

        """
        self.account_name = account_name

    def get_account_name(self):
        """Get account name.

        Returns:
            str: Account name.

        """
        return self.account_name

    def set_paid_through_account_name(self, paid_through_account_name):
        """Set paid through account name.

        Args:
            paid_through_account_name(str): Paid through account name.

        """
        self.paid_through_account_name = paid_through_account_name

    def get_paid_through_account_name(self):
        """Get paid through account name.

        Returns:
            str: Paid through account name.

        """
        return self.paidf_through_account_name

    def set_vendor_name(self, vendor_name): 
        """Set vendor name.

        Args:
            vendor_name(str): Vendor name.

        """
        self.vendor_name = vendor_name

    def get_vendor_name(self):
        """Get vendor name.

        Returns:
            str: Vendor name.
  
        """
        return self.vendor_name

    def set_tax_name(self, tax_name):
        """Set tax name.

        Args:
            tax_name(str): Tax name.

        """
        self.tax_name = tax_name

    def get_tax_name(self):
        """Get tax name.
 
        Returns:
            str: Tax name.

        """
        return self.tax_name

    def set_tax_percentage(self, tax_percentage):
        """Set tax percentage.

        Args:
            tax_percentage(float): Tax percentage.

        """
        self.tax_percentage = tax_percentage

    def get_tax_percentage(self): 
        """Get tax percentage.

        Returns:
            float: Tax percentage.

        """
        return self.tax_percentage

    def set_currency_code(self, currency_code):
        """Set currency code.

        Args:
            currency_code(str): Currency code.

        """ 
        self.currency_code = currency_code

    def get_currency_code(self):
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code

    def set_tax_amount(self, tax_amount):
        """Set tax amount.

        Args:
            tax_amount(float): Tax amount.

        """
        self.tax_amount = tax_amount

    def get_tax_amount(self):
        """Get tax amount.

        Returns:
            float: Tax amount.

        """
        return self.tax_amount

    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.

        """
        self.sub_total = sub_total

    def get_sub_total(self): 
        """Get sub total.

        Returns:
            float: Sub total.
 
        """
        return self.sub_total

    def set_total(self, total):
        """Set total.

        Args:
            total(float): Total.

        """
        self.total = total
  
    def get_total(self):
        """Get total.

        Returns:
            float: Total.

        """
        return self.total

    def set_bcy_total(self, bcy_total):
        """Set bcy total.

        Args:
            bcy_total(float): Bcy total.

        """
        self.bcy_total = bcy_total

    def get_bcy_total(self):
        """Get bcy total.

        Returns:
            float: Bcy total.

        """
        return self.bcy_total

    def set_reference_number(self, reference_number):
        """Set reference number.

        Args:
            reference_number(str): Reference number.

        """
        self.reference_number = reference_number

    def get_reference_number(self):
        """Get reference_number.

        Returns:
            str: Reference number.

        """
        return self.reference_number

    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description

        """
        self.description = description

    def get_description(self): 
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_customer_name(self, customer_name): 
        """Set customer name.

        Args: 
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_expense_receipt_name(self, expense_receipt_name):
        """Set expense receipt name.

        Args:
            expense_receipt_name(str): Expense receipt name.

        """
        self.expense_receipt_name = expense_receipt_name

    def get_expense_receipt_name(self):
        """Get expense receipt name.

        Returns:
            str: Expense receipt name.

        """
        return self.expense_receipt_name

    def set_created_time(self, created_time): 
        """Set created time.

        Args:
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self):
        """Get created time.

        Returns:
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args:
            last_modified_time(str): Last modified time.

        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: Last modified time.

        """
        return self.last_modified_time

    def set_status(self, status):
        """Set status.

        Args:
            status(str): Status.

        """
        self.status = status

    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return status

    def set_invoice_id(self, invoice_id):
        """Set invoice id.

        Args:
            invoice_id(str): Invoice id.

        """
        self.invoice_id = invoice_id

    def get_invoice_id(self):
        """Get invoice id.

        Returns:
            str: Invoice id.

        """

    def set_invoice_number(self, invoice_number): 
        """Set invoice number.

        Args:
            invoice_number(str): Invoice number.

        """
        self.invoice_number = invoice_number

    def get_invoice_number(self): 
        """Get invoice number.

        Returns:
            str: Invoice number.

        """
        return self.invoice_number

    def set_project_name(self, project_name): 
        """Set project name.

        Args:
            project_name(str): Project name.

        """
        self.project_name = project_name

    def get_project_name(self):
        """Get project name.

        Returns:
            str: Project name.

        """
        return self.project_name

    def set_recurring_expense_id(self, recurring_expense_id):
        """Set recurring expense id.

        Args:
            recurring_expense_id(str): Recurring expense id.

        """
        self.recurring_expense_id = recurring_expense_id

    def get_recurring_expense_id(self):
        """Get recurring expense id.

        Returns:
            str: Recurring expense id.

        """
        return self.recurring_expense_id

    def to_json(self):
        """This method is used to convert expense object to json object.

        Returns:
            dict: Dictionary containing json object for expenses.

        """
        data = {}
        if self.account_id != '':
            data['account_id'] = self.account_id
        if self.paid_through_account_id != '':
            data['paid_through_account_id'] = self.paid_through_account_id
        if self.date != '':
            data['date'] = self.date
        if self.amount > 0:
            data['amount'] = self.amount
        if self.tax_id != '':
            data['tax_id'] = self.tax_id
        if self.is_inclusive_tax is not None:
            data['is_inclusive_tax'] = self.is_inclusive_tax
        if self.reference_number != '':
            data['reference_number'] = self.reference_number
        if self.description != '':
            data['description'] = self.description
        if self.is_billable is not None:
            data['is_billable'] = self.is_billable
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.vendor_id != '':
            data['vendor_id'] = self.vendor_id
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        if self.recurring_expense_id != '':
            data['recurring_expense_id'] = self.recurring_expense_id
        if self.project_id != '':
            data['project_id'] = self.project_id
        if self.product_type != '':
            data['product_type'] = self.product_type
        if self.hsn_or_sac != '':
            data['hsn_or_sac'] = self.hsn_or_sac
        if self.gst_no != '':
            data['gst_no'] = self.gst_no
        if self.gst_treatment != '':
            data['gst_treatment'] = self.gst_treatment
        if self.destination_of_supply != '':
            data['destination_of_supply'] = self.destination_of_supply
        return data







#$Id$

class RecurringExpense:
    """This class is used to create object for Recurring Expenses."""
    def __init__(self):
        """Initialize parameters for Recurring expenses."""
        self.account_id = ''
        self.paid_through_account_id = ''
        self.recurrence_name = ''
        self.start_date = ''
        self.end_date = ''
        self.recurrence_frequency = ''
        self.repeat_every = 0
        self.repeat_frequency = ''
        self.amount = 0.0
        self.tax_id = ''
        self.is_inclusive_tax = None
        self.is_billable = None
        self.customer_id = ''
        self.vendor_id = ''
        self.project_id = ''
        self.currency_id = ''
        self.exchange_rate = 0.0
        self.recurring_expense_id = ''
        self.last_created_date = ''
        self.next_expense_date = ''
        self.account_name = ''
        self.paid_through_account_name = ''
        self.vendor_name = ''
        self.currency_code = ''
        self.tax_name = ''
        self.tax_percentage = 0
        self.tax_amount = 0.0
        self.sub_total = 0.0
        self.total = 0.0
        self.bcy_total = 0.0
        self.description = ''
        self.customer_name = ''
        self.status = ''
        self.created_time = ''
        self.last_modified_time = ''
        self.project_name = ''
  
    def set_account_id(self, account_id):
        """Set account id.

        Args:
            account_id(str): Account id.

        """
        self.account_id = account_id

    def get_account_id(self):
        """Get account id.

        Returns:
            str: Account id.

        """
        return self.account_id
  
    def set_paid_through_account_id(self, paid_through_account_id):
        """Set paid through account id.

        Args:
            paid_through_account_id(str): Paid through account id.

        """
        self.paid_through_account_id = paid_through_account_id

    def get_paid_through_account_id(self):
        """Get paid through account id.

        Returns:
            str: Paid through account id.

        """
        return self.paid_through_account_id

    def set_recurrence_name(self, recurrence_name):
        """Set recurrence name.

        Args:
            recurrence_name(str): Recurrence name.

        """
        self.recurrence_name = recurrence_name

    def get_recurrence_name(self):
        """Get recurrence name.

        Returns: 
            str: Recurrence name.

        """
        return self.recurrence_name

    def set_start_date(self, start_date):
        """Set start date.

        Args:
            start_date(str): Start date.

        """
        self.start_date = start_date

    def get_start_date(self):
        """Get start date.

        Returns:
            str: Start date.

        """
        return self.start_date
  
    def set_end_date(self, end_date):
        """Set end date.

        Args:
            end_date(str): End date.

        """
        self.end_date = end_date

    def get_end_date(self):
        """Get end date.

        Returns:
            str: End date.

        """
        return self.end_date
  
    def set_recurrence_frequency(self, recurrence_frequency):
        """Set recurrence frequency.

        Args:
            recurrence_frequency(str): Recurrence frequency.

        """
        self.recurrence_frequency = recurrence_frequency

    def get_recurrence_frequency(self):
        """Get recurrence frequency.

        Returns:
            str: Recurrence frequency.

        """
        return self.recurrence_frequency
  
    def set_repeat_every(self, repeat_every):
        """Set repeat every.
    
        Args:
            repeat_every(int): Repeat every.

        """
        self.repeat_every = repeat_every

    def get_repeat_every(self):
        """Get repeat every.

        Returns:
            int: Repeat every.
 
        """
        return self.repeat_every
 
    def set_repeat_frequency(self, repeat_frequency):
        """Set repeat frequency.
 
        Args:
            repeat_frequency(str): Repeat frequency.

        """
        self.repeat_frequency = repeat_frequency

    def get_repeat_frequency(self):
        """Get repeat frequency.

        Returns:
            str: Repeat frequency.

        """
        return self.repeat_frequency
 
    def set_amount(self, amount):
        """Set amount.
 
        Args:
            amount(float): Amount.

        """
        self.amount = amount

    def get_amount(self):
        """Get amount.

        Returns:
            float: Amount.

        """
        return self.amount

    def set_tax_id(self, tax_id):
        """Set tax id.

        Args:
            tax_id(str): Tax id.

        """
        self.tax_id = tax_id

    def get_tax_id(self): 
        """Get tax id.

        Returns:
            str: Tax id.

        """
        return self.tax_id

    def set_is_inclusive_tax(self, is_inclusive_tax):
        """Set whether tax is inclusive.

        Args:
            is_inclusive_tax(bool): True if tax is inclusive else False.

        """
        self.is_inclusive_tax = is_inclusive_tax

    def get_is_inclusive_tax(self):
        """Get whether tax is inclusive.

        Returns:
            bool: True if tax is inclusive else False.

        """
        return self.is_inclusive_tax
  
    def set_is_billable(self, is_billable):
        """Set whether Recurring Expense is billable.

        Args:
            is_billable(bool): True if billable else False.

        """ 
        self.is_billable = is_billable
    
    def get_is_billable(self):
        """Get whether Recurring Expense is billable.

        Returns:
            bool: True if billable else False.

        """
        return self.is_billable
  
    def set_customer_id(self, customer_id): 
        """Set customer id.

        Args:
            customer_id(str): Customer id.

        """
        self.customer_id = customer_id

    def get_customer_id(self): 
        """Get customer id.

        Returns:
            str: Customer id.

        """
        return self.customer_id
   
    def set_vendor_id(self, vendor_id):
        """Set vendor id.

        Args:
            vendor_id(str): Vendor id.

        """
        self.vendor_id = vendor_id

    def get_vendor_id(self):
        """Get vendor id.

        Returns:
            str: Get vendor id.

        """
        return self.vendor_id

    def set_project_id(self, project_id):
        """Set project id.

        Args:
            project_id(str): Project id.

        """ 
        self.project_id = project_id

    def get_project_id(self):
        """Get project id.

        Returns:
            str: Project id.

        """
        return self.project_id
  
    def set_currency_id(self, currency_id):
        """Set currency id.

        Args:
            currency_id(str): Currency id.

        """
        self.currency_id = currency_id
 
    def get_currency_id(self):
        """Get currency id.

        Returns:
            str: Currency id.

        """
        return self.currency_id

    def set_exchange_rate(self, exchange_rate):
        """Set exchange rate.

        Args:
            exchange_rate(float): Exchange rate.

        """          
        self.exchange_rate = exchange_rate
 
    def get_exchange_rate(self):
        """Get exchange rate.

        Returns:
            float: Exchange rate.

        """
        return self.exchange_rate
  
    def set_recurring_expense_id(self, recurring_expense_id): 
        """Set recurring expense id.
 
        Args:
            recurring_expense_id(str): Recurring expense id.
 
        """
        self.recurring_expense_id = recurring_expense_id

    def get_recurring_expense_id(self): 
        """Get recurring expense id.

        Returns:
            str: Recurring expense id.

        """
        return self.recurring_expense_id

    def set_last_created_date(self, last_created_date):
        """Set last created date.

        Args:
            last_created_date(str): Last created date.

        """
        self.last_created_date = last_created_date

    def get_last_created_date(self): 
        """Get last created date.

        Returns:
            str: Last created date.

        """
        return self.last_created_date

    def set_next_expense_date(self, next_expense_date): 
        """Set next expense date.

        Args:
            next_expense_date(str): Next expense date.

        """
        self.next_expense_date = next_expense_date

    def get_next_expense_date(self): 
        """Get next expense date.

        Returns:
            str: Next expense date.

        """
        return self.next_expense_date

    def set_account_name(self, account_name):
        """Set account name.

        Args:
            account_name(str): Account name.

        """
        self.account_name = account_name

    def get_account_name(self): 
        """Get account name.

        Return:
            str: Account name.

        """
        return self.account_name

    def set_paid_through_account_name(self, paid_through_account_name):
        """Set paid through account name.

        Args:
            paid_through_account_name(str): Paid through account name.

        """
        self.paid_through_account_name = paid_through_account_name

    def get_paid_through_account_name(self):
        """Get paid through account name.

        Returns:
            str: Paid through account name.
  
        """
        return self.paid_through_account_name

    def set_vendor_name(self, vendor_name):
        """Set vendor name.

        Args:
            vendor_name(str): Vendor name.

        """
        self.vendor_name = vendor_name

    def get_vendor_name(self):
        """Get vendor name.

        Returns:
            str: Vendor name.

        """
        return self.vendor_name

    def set_currency_code(self, currency_code): 
        """Set currency code.

        Args:
            currency_code(str): Currency code.

        """
        self.currency_code = currency_code

    def get_currency_code(self): 
        """Get currency code.

        Returns:
            str: Currency code.

        """
        return self.currency_code

    def set_tax_name(self, tax_name):
        """Set tax name.

        Args:
            tax_name(str): Tax name.

        """
        self.tax_name = tax_name

    def get_tax_name(self):
        """Get tax name.

        Returns:
            str: Tax name.

        """
        return self.tax_name

    def set_tax_percentage(self, tax_percentage):
        """Set tax percentage.

        Args:
            tax_percentage(int): Tax percentage.

        """
        self.tax_percentage = tax_percentage

    def set_tax_amount(self, tax_amount):
        """Set tax amount.

        Args:
            tax_amount(float): Tax amount.

        """
        self.tax_amount = tax_amount

    def get_tax_amount(self): 
        """Get tax amount.

        Returns:
            float: Tax amount.

        """
        return self.tax_amount

    def set_sub_total(self, sub_total):
        """Set sub total.

        Args:
            sub_total(float): Sub total.

        """
        self.sub_total = sub_total

    def get_sub_total(self): 
        """Get sub total.

        Returns:
            float: Sub total.

        """
        return self.sub_total

    def set_total(self, total):
        """Set total.

        Args:
            total(float): Total.
 
        """    
        self.total = total

    def get_total(self): 
        """Get total.

        Returns:
            float: Total.

        """
        return self.total

    def set_bcy_total(self, bcy_total):
        """Set bcy total.

        Args:
            bcy_total(float): Bcy total.

        """
        self.bcy_total = bcy_total

    def get_bcy_total(self):
        """Get bcy total.

        Returns:
            float: Bcy total.

        """
        return self.bcy_total

    def set_description(self, description):
        """Set description.

        Args:
            description(str): Description.

        """
        self.description = description

    def get_description(self): 
        """Get description.

        Returns:
            str: Description.

        """
        return self.description

    def set_customer_name(self, customer_name):
        """Set customer name.
     
        Args:
            customer_name(str): Customer name.

        """
        self.customer_name = customer_name

    def get_customer_name(self):
        """Get customer name.

        Returns:
            str: Customer name.

        """
        return self.customer_name

    def set_status(self, status): 
        """Set status.
 
        Args:
            status(str): Status.

        """
        self.status = ''
 
    def get_status(self):
        """Get status.

        Returns:
            str: Status.

        """
        return self.status

    def set_created_time(self, created_time):
        """Set created time.

        Args: 
            created_time(str): Created time.

        """
        self.created_time = created_time

    def get_created_time(self): 
        """Get created time.

        Returns:
            str: Created time.

        """
        return self.created_time

    def set_last_modified_time(self, last_modified_time):
        """Set last modified time.

        Args:
            last_modified_time(str): Last modified time.
 
        """
        self.last_modified_time = last_modified_time

    def get_last_modified_time(self):
        """Get last modified time.

        Returns:
            str: Last modified time.

        """
        return self.last_modified_time

    def set_project_name(self, project_name):
        """Set project name.

        Args:
            project_name(str): Project name.

        """
        self.project_name = project_name

    def get_project_name(self): 
        """Get project name.

        Returns:
            str: Project name.

        """
        return self.project_name

    def to_json(self):
        """This method is used to convert recurring expenses object to json object.

        Returns:
            dict: Dictionary coontaining json object for recurring expenses.

        """
        data = {}
        if self.account_id != '':
            data['account_id'] = self.account_id
        if self.paid_through_account_id != '':
            data['paid_through_account_id'] = self.paid_through_account_id
        if self.recurrence_name != '':
            data['recurrence_name'] = self.recurrence_name
        if self.start_date != '':
            data['start_date'] = self.start_date
        if self.end_date != '':
            data['end_date'] = self.end_date
        if self.recurrence_frequency != '':  
            data['recurrence_frequency'] = self.recurrence_frequency
        if self.repeat_every > 0:
            data['repeat_every'] = self.repeat_every
        if self.amount > 0:
            data['amount'] = self.amount
        if self.tax_id != '':
            data['tax_id'] = self.tax_id
        if self.is_inclusive_tax is not None:
            data['is_inclusive_tax'] = self.is_inclusive_tax
        if self.is_billable is not None:
            data['is_billable'] = self.is_billable
        if self.customer_id != '':
            data['customer_id'] = self.customer_id
        if self.vendor_id != '':
            data['vendor_id'] = self.vendor_id
        if self.project_id != '':
            data['project_id'] = self.project_id
        if self.currency_id != '':
            data['currency_id'] = self.currency_id
        if self.exchange_rate > 0:
            data['exchange_rate'] = self.exchange_rate
        return data





#$Id$#

class BooksException(Exception):
    """This class is used to create object for Books Exception."""
  
    def __init__(self,content):
        """Initialize parameters for books exception.
   
        Args: 
            content(dict): Response content.

        """
        Exception.__init__(self)
        print(content)
        self.code = str(content['code'])
        self.message = content['message']

    def set_code(self, code):
        """Set code.
       
        Args: 
            code(int): Error code.
  
        """
        self.code = code

    def get_code(self):
        """Get code.
 
        Returns:
            int: Error code.

        """
        return self.code
  
    def set_message(self, message):
        """Set message.

        Args:
            message(str): Error message.
 
        """
        self.message = message

    def get_message(self):
        """Get message.
       
        Returns:
            str: Error message.
 
        """
        return self.message

